from typing import List

from . import DIMap
from ....model.types.data_type import DataItem
from ....common.transform import pad_with_zeros


def init_parameter_def(ParaMeterTypes: List[DataItem]):
    """初始化参变量定义
    
    Args:
        ParaMeterTypes: 参变量类型列表，包含参变量类型
    """
    for data_type in ParaMeterTypes:
        di = int(data_type.di)
        # 时段表数据
        if 0x04010000 <= di <= 0x04020008:
            if data_type.di not in DIMap:
                DIMap[data_type.di] = []
            data_item = DataItem(
                di=data_type.di,
                name=data_type.name,
                data_format=data_type.data_format,
                value=pad_with_zeros(len(data_type.data_format)),
                unit=data_type.unit,
            )
            DIMap[data_type.di].append(data_item)
        else:
            DIMap[data_type.di] = DataItem(
                di=data_type.di,
                name=data_type.name,
                data_format=data_type.data_format,
                value=pad_with_zeros(len(data_type.data_format)),
                unit=data_type.unit,
            )



"""DLT645 客户端服务模块。

本模块实现了 DLT645 协议的客户端业务服务功能，包括：
- 电能数据读取
- 需量数据读取
- 变量数据读取
- 事件记录读取
- 参变量读写
- 通讯地址读写
- 密码管理
"""

from concurrent.futures import ThreadPoolExecutor
from datetime import datetime
import time
import struct
from typing import Optional, Union

from ...common.transform import (
    bcd_to_float,
    bcd_to_time,
    bcd_to_string,
    bytes_to_int,
    bytes_to_spaced_hex,
    string_to_bcd,
)
from ...model.validators import validate_device
from ...model.types.data_type import DataFormat, DataItem
from ...model.types.dlt645_type import (
    DI_LEN,
    ADDRESS_LEN,
    PASSWORD_LEN,
    CtrlCode,
    Demand,
    EventRecord,
    PasswordManager,
    ErrorCode,
    get_error_msg,
)
from ...protocol.protocol import DLT645Protocol
from ...protocol.frame import Frame
from ...model.data import data_handler as data
from ...service.clientsvc.log import log
from ...transport.client.rtu_client import RtuClient
from ...transport.client.tcp_client import TcpClient
from ...common.message_capture import MessageCapture
from ...common.message_types import MessageRecord, MessagePair
from typing import List


class MeterClientService:
    """电表客户端服务类。

    用于与 DLT645 电表设备进行通信，提供数据读写等业务功能。

    :ivar address: 设备地址（6字节）。
    :ivar password_manager: 密码管理器。
    :ivar operation_code: 操作码（4字节）。
    :ivar client: 通信客户端（TCP 或 RTU）。
    """

    def __init__(self, client: Union[TcpClient, RtuClient]):
        """初始化电表客户端服务。

        :param client: 通信客户端实例（TcpClient 或 RtuClient）。
        :type client: Union[TcpClient, RtuClient]
        """
        self.address = bytearray(6)  # 6字节地址
        self.password_manager: PasswordManager = PasswordManager()  # 4字节密码
        self.operation_code = bytearray(4)  # 4字节操作码
        self.client = client
        self._executor = ThreadPoolExecutor(max_workers=1)  # 用于超时控制

    @classmethod
    def new_tcp_client(
        cls, ip: str, port: int, timeout: float = 30.0
    ) -> Optional["MeterClientService"]:
        """创建TCP客户端"""
        tcp_client = TcpClient(ip=ip, port=port, timeout=timeout)

        # 创建业务服务实例
        return cls.new_meter_client_service(tcp_client)

    @classmethod
    def new_rtu_client(
        cls,
        port: str,
        baudrate: int,
        databits: int,
        stopbits: int,
        parity: str,
        timeout: float,
    ) -> Optional["MeterClientService"]:
        """创建RTU客户端"""
        rtu_client = RtuClient(
            port=port,
            baud_rate=baudrate,
            data_bits=databits,
            stop_bits=stopbits,
            parity=parity,
            timeout=timeout,
        )

        # 创建业务服务实例
        return cls.new_meter_client_service(rtu_client)

    @classmethod
    def new_meter_client_service(
        cls, client: Union[TcpClient, RtuClient]
    ) -> Optional["MeterClientService"]:
        """创建新的MeterService实例"""
        service = cls(client)
        return service

    def get_time(self, t: bytes) -> datetime:
        """从字节数据获取时间"""
        timestamp = bytes_to_int(t)
        log.debug(f"timestamp: {timestamp}")
        return datetime.fromtimestamp(timestamp)

    def set_address(self, address: str) -> bool:
        """设置设备地址"""
        address = string_to_bcd(address)
        if len(address) != ADDRESS_LEN:
            log.error("无效的地址长度")
            return False

        self.address = address
        log.info(f"设置客户端通讯地址: {bytes_to_spaced_hex(self.address)}")
        return True

    def set_password(self, password: str) -> bool:
        """设置设备密码, 修改数据的命令会带上密码发送出去"""
        password = string_to_bcd(password)
        self.password_manager.set_password(password)
        log.info(f"设置客户端密码: {bytes_to_spaced_hex(password)}")
        return True

    def change_password(self, old_password: str, new_password: str) -> bool:
        """修改设备密码"""
        old_password = string_to_bcd(old_password)
        new_password = string_to_bcd(new_password)
        if not self.password_manager.is_password_valid(old_password):
            return False
        if not self.password_manager.is_password_valid(new_password):
            return False

        new_level = new_password[0]
        di = 0x40000C00 | new_level  # 新密码的DI
        write_data = old_password + new_password
        data_bytes = struct.pack("<I", di) + write_data  # 小端序
        frame_bytes = DLT645Protocol.build_frame(
            self.address, CtrlCode.ChangePassword, data_bytes
        )
        return self.send_and_handle_request(frame_bytes)

    def read_00(self, di: int) -> Optional[DataItem]:
        """读取电能"""
        data_bytes = struct.pack("<I", di)  # 小端序
        frame_bytes = DLT645Protocol.build_frame(
            self.address, CtrlCode.ReadData, data_bytes
        )
        return self.send_and_handle_request(frame_bytes)

    def read_01(self, di: int) -> Optional[DataItem]:
        """读取最大需量及发生时间"""
        data_bytes = struct.pack("<I", di)  # 小端序
        frame_bytes = DLT645Protocol.build_frame(
            self.address, CtrlCode.ReadData, data_bytes
        )
        return self.send_and_handle_request(frame_bytes)

    def read_02(self, di: int) -> Optional[DataItem]:
        """读取变量"""
        data_bytes = struct.pack("<I", di)  # 小端序
        frame_bytes = DLT645Protocol.build_frame(
            self.address, CtrlCode.ReadData, data_bytes
        )
        return self.send_and_handle_request(frame_bytes)

    def read_03(self, di: int) -> Optional[DataItem]:
        """读取事件记录"""
        data_bytes = struct.pack("<I", di)  # 小端序
        frame_bytes = DLT645Protocol.build_frame(
            self.address, CtrlCode.ReadData, data_bytes
        )
        return self.send_and_handle_request(frame_bytes)

    def read_04(self, di: int) -> Optional[DataItem]:
        """读取参变量"""
        data_bytes = struct.pack("<I", di)  # 小端序
        frame_bytes = DLT645Protocol.build_frame(
            self.address, CtrlCode.ReadData, data_bytes
        )
        return self.send_and_handle_request(frame_bytes)

    def write_04(self, di: int, value: str, password: str) -> Optional[DataItem]:
        """写参变量"""
        # 密码 + 操作码 + 值
        password = string_to_bcd(password)
        write_data = (
            password + self.operation_code + string_to_bcd(value, endian="little")
        )
        data_bytes = struct.pack("<I", di) + write_data  # 小端序
        frame_bytes = DLT645Protocol.build_frame(
            self.address, CtrlCode.WriteData, data_bytes
        )
        return self.send_and_handle_request(frame_bytes)

    def read_address(self) -> Optional[DataItem]:
        """读取通讯地址"""
        # 读取通讯地址需要使用特殊的广播地址0xAAAAAAAAAAAA
        broadcast_address = bytearray([0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA])
        frame_bytes = DLT645Protocol.build_frame(
            broadcast_address, CtrlCode.ReadAddress, None
        )
        return self.send_and_handle_request(frame_bytes)

    def write_address(self, new_address: bytes) -> Optional[DataItem]:
        """写通讯地址"""
        if len(new_address) != ADDRESS_LEN:
            log.error("无效的新地址长度")
            return None

        frame_bytes = DLT645Protocol.build_frame(
            self.address, CtrlCode.WriteAddress, new_address
        )
        return self.send_and_handle_request(frame_bytes)

    def send_and_handle_request(
        self,
        frame_bytes: bytes,
    ) -> Optional[DataItem]:
        """发送请求并处理响应（带超时控制）

        Args:
            frame_bytes: 要发送的帧数据

        Returns:
            DataItem: 成功时返回数据项
            None: 超时或失败时返回
        """
        try:
            if self.client is None:
                log.error("连接未初始化")
                return None

            # 确保连接有效（重用连接，如果断开则重新连接）
            if not self.client._ensure_connection():
                log.error("连接失败")
                return None

            # 请求阶段超时控制
            response = self.client.send_request(frame_bytes)

            if response is None:
                return None

            # 解析阶段
            frame = DLT645Protocol.deserialize(response)
            if frame is None:
                log.error("解析响应失败")
                return None

            # 处理响应
            data_item = self.handle_response(frame)
            return data_item
        except Exception as e:
            log.error(f"未知错误: {str(e)}", exc_info=True)
            return None

    def _is_valid_response(self, frame: Frame) -> bool:
        """验证响应帧是否有效"""
        # 检测异常处理帧 (DLT645协议中，异常响应的控制码次高位为1)
        if (frame.ctrl_code & 0x40) == 0x40:  # 检查次高位
            error_code = frame.data[0] if len(frame.data) > 0 else None
            error_msg = "设备返回异常响应"

            # 如果数据域不为空，尝试解析错误码
            if frame.data:
                error_code = frame.data[0] if len(frame.data) > 0 else None
                # 根据常见的DLT645错误码定义错误信息
                if any(error_code == ec for ec in ErrorCode):
                    error_msg = f"设备返回异常响应: {get_error_msg(error_code)} (错误码: {error_code:02X})"
                else:
                    error_msg = f"设备返回异常响应: 未知错误码"

            log.error(error_msg)
            return False
        return True

    def handle_response(self, frame: Frame) -> Optional[DataItem]:
        """处理响应帧，包括异常帧检测"""
        try:
            if not self._is_valid_response(frame):
                return None

            # 验证设备地址 - 特殊控制码不需要验证
            if not validate_device(self.address, frame.ctrl_code, frame.addr):
                log.warning(f"验证设备地址: {bytes_to_spaced_hex(frame.addr)} 失败")
                return None

            # 根据控制码判断响应类型
            if frame.ctrl_code == (CtrlCode.BroadcastTimeSync | 0x80):  # 广播校时响应
                log.debug(f"广播校时响应: {bytes_to_spaced_hex(frame.data)}")
                time_value = self.get_time(frame.data[0:4])
                data_item = data.get_data_item(bytes_to_int(frame.data[0:4]))
                if not data_item:
                    log.warning("获取数据项失败")
                    return None
                data_item.value = time_value
                return data_item

            elif frame.ctrl_code == (CtrlCode.ReadData | 0x80):  # 读数据响应
                # 解析数据标识
                if len(frame.data) < DI_LEN:
                    log.warning("读数据响应数据长度无效")
                    return None

                di = frame.data[0:DI_LEN]
                di3 = di[3]

                if di3 == 0x00:  # 读取电能响应
                    log.debug(f"读取电能响应: {bytes_to_spaced_hex(frame.data)}")
                    data_item = data.get_data_item(bytes_to_int(di))
                    if not data_item:
                        log.error("获取电能数据项失败")
                        return None
                    data_item.value = bcd_to_float(
                        frame.data[4:8], data_item.data_format, "little"
                    )
                    return data_item

                elif di3 == 0x01:  # 读取最大需量及发生时间响应
                    log.debug(
                        f"读取最大需量及发生时间响应: {bytes_to_spaced_hex(frame.data)}"
                    )
                    data_item = data.get_data_item(bytes_to_int(di))
                    if not data_item:
                        log.error("获取最大需量数据项失败")
                        return None

                    # 转换时间
                    occur_time = bcd_to_time(frame.data[7:12])

                    # 转换需量值
                    demand_value = bcd_to_float(
                        frame.data[DI_LEN : DI_LEN + 3], data_item.data_format, "little"
                    )

                    data_item.value = Demand(value=demand_value, time=occur_time)
                    return data_item

                elif di3 == 0x02:
                    data_item = data.get_data_item(bytes_to_int(di))
                    if not data_item:
                        log.error("获取变量数据项失败")
                        return None
                    data_item.value = bcd_to_float(
                        frame.data[DI_LEN : DI_LEN + 4], data_item.data_format, "little"
                    )
                    return data_item
                elif di3 == 0x03:
                    log.debug(f"读取事件记录响应: {bytes_to_spaced_hex(frame.data)}")
                    data_item = data.get_data_item(bytes_to_int(di))
                    if not data_item:
                        log.error("获取事件记录数据项失败")
                        return None

                    start_len = DI_LEN
                    for i, item in enumerate(data_item):
                        event_record: EventRecord = item.value
                        if isinstance(event_record.event, tuple):
                            step = len(item.data_format.split(",")[0]) // 2
                            event_list = list(event_record.event)
                            for i, _ in enumerate(event_list):
                                # 提取BCD数据部分
                                bcd_data = frame.data[start_len : start_len + step]
                                start_len += step
                                event_list[i] = bcd_to_string(bcd_data, "little")
                            event_record.event = tuple(reversed(event_list))
                        else:
                            step = len(item.data_format) // 2
                            # 提取BCD数据部分
                            bcd_data = frame.data[start_len : start_len + step]
                            start_len += step
                            item.value = bcd_to_string(bcd_data, "little")
                    return data_item
                elif di3 == 0x04:  # 读参变量响应
                    log.debug(f"读取参变量响应: {bytes_to_spaced_hex(frame.data)}")
                    data_item = data.get_data_item(bytes_to_int(di))
                    if not data_item:
                        log.error("获取参变量数据项失败")
                        return None

                    start_len = DI_LEN
                    # 时段表数据
                    if (
                        0x04010000
                        <= int.from_bytes(di, byteorder="little")
                        <= 0x04020008
                    ):
                        for i, item in enumerate(data_item):
                            step = len(item.data_format) // 2
                            # 提取BCD数据部分
                            bcd_data = frame.data[start_len : start_len + step]
                            start_len += step
                            item.value = bcd_to_string(bcd_data, "little")
                    else:
                        # 提取BCD数据部分
                        bcd_data = frame.data[start_len:]
                        start_len += len(bcd_data)
                        data_item.value = bcd_to_string(bcd_data, "little")
                    return data_item
                else:
                    log.warning(f"未知数据项: {bytes_to_spaced_hex(di)}")
                    return None
            elif frame.ctrl_code == (CtrlCode.WriteData | 0x80):  # 写参变量响应
                log.debug(f"写参变量")
                return None
            elif frame.ctrl_code == (CtrlCode.ReadAddress | 0x80):  # 读通讯地址响应
                log.debug(f"读通讯地址响应: {bytes_to_spaced_hex(frame.data)}")
                if len(frame.data) == ADDRESS_LEN:
                    self.address = frame.data[:ADDRESS_LEN]
                return DataItem(
                    di=bytes_to_int(frame.data[0:DI_LEN]),
                    name="通讯地址",
                    data_format=DataFormat.XXXXXXXX.value,
                    value=bcd_to_string(frame.data),
                    unit="",
                    update_time=datetime.now(),
                )

            elif frame.ctrl_code == (CtrlCode.WriteAddress | 0x80):  # 写通讯地址响应
                log.debug(f"写通讯地址响应: {bytes_to_spaced_hex(frame.data)}")
                return DataItem(
                    di=bytes_to_int(frame.data[0:DI_LEN]),
                    name="通讯地址",
                    data_format=DataFormat.XXXXXXXX.value,
                    value=bcd_to_string(frame.data),
                    unit="",
                    update_time=datetime.now(),
                )
            elif frame.ctrl_code == (CtrlCode.ChangePassword | 0x80):  # 写密码响应
                log.debug(f"写密码响应: {bytes_to_spaced_hex(frame.data)}")
                password = frame.data[:DI_LEN]
                self.password_manager.set_password(password)
            else:
                log.warning(f"Unknown control code: {frame.ctrl_code}")
                return None
        except Exception as e:
            log.error(f"处理响应帧时出错: {e}")
            raise

    # ==================== 报文捕获方法 ====================

    def enable_message_capture(self, queue_size: int = 100) -> None:
        """启用报文捕获功能。

        :param queue_size: 报文队列大小，默认100
        :type queue_size: int
        """
        if self.client._message_capture is None:
            self.client._message_capture = MessageCapture(enabled=True, queue_size=queue_size)
        else:
            self.client._message_capture.enable()
            self.client._message_capture.set_queue_size(queue_size)
        log.info(f"报文捕获已启用，队列大小: {queue_size}")

    def disable_message_capture(self) -> None:
        """禁用报文捕获功能。"""
        if self.client._message_capture:
            self.client._message_capture.disable()
        log.info("报文捕获已禁用")

    def get_captured_messages(self, count: int = 0) -> List[MessageRecord]:
        """获取捕获的报文列表。

        :param count: 要获取的数量，0表示全部
        :type count: int
        :return: 报文列表
        :rtype: List[MessageRecord]
        """
        if self.client._message_capture:
            return self.client._message_capture.get_all_messages(count)
        return []

    def get_captured_tx_messages(self, count: int = 0) -> List[MessageRecord]:
        """获取捕获的发送报文列表。

        :param count: 要获取的数量，0表示全部
        :type count: int
        :return: 发送报文列表
        :rtype: List[MessageRecord]
        """
        if self.client._message_capture:
            return self.client._message_capture.get_tx_messages(count)
        return []

    def get_captured_rx_messages(self, count: int = 0) -> List[MessageRecord]:
        """获取捕获的接收报文列表。

        :param count: 要获取的数量，0表示全部
        :type count: int
        :return: 接收报文列表
        :rtype: List[MessageRecord]
        """
        if self.client._message_capture:
            return self.client._message_capture.get_rx_messages(count)
        return []

    def get_captured_pairs(self, count: int = 0) -> List[MessagePair]:
        """获取捕获的TX/RX配对列表。

        :param count: 要获取的数量，0表示全部
        :type count: int
        :return: 配对列表
        :rtype: List[MessagePair]
        """
        if self.client._message_capture:
            return self.client._message_capture.get_pairs(count)
        return []

    def clear_captured_messages(self) -> None:
        """清空所有捕获的报文。"""
        if self.client._message_capture:
            self.client._message_capture.clear()
        log.info("捕获的报文已清空")

    def get_message_capture_stats(self) -> dict:
        """获取报文捕获统计信息。

        :return: 统计信息字典
        :rtype: dict
        """
        if self.client._message_capture:
            return self.client._message_capture.get_stats()
        return {"enabled": False}



import time

from src.service.serversvc.server_service import MeterServerService

if __name__ == '__main__':
    server_svc = MeterServerService.new_tcp_server("127.0.0.1", 10521, 3000)
    # server_svc = MeterServerService.new_rtu_server("COM11", 8, 1, 2400, "E", 1000)
    server_svc.set_00(0x00000000, 100.0)
    server_svc.set_02(0x02010100, 86.0)
    server_svc.enable_message_capture(queue_size=50)
    server_svc.server.start()
    while True:
        messages = server_svc.get_captured_pairs()
        for message in messages:
            print(message)
        time.sleep(1)


from typing import List

from . import DIMap
from ....model.types.data_type import DataItem
from ....model.types.dlt645_type import EventRecord
from ....common.transform import pad_with_zeros


def init_event_record_def(EventRecordTypes: List[DataItem]):
    """初始化事件记录定义
    
    Args:
        EventRecordTypes: 事件记录类型列表，包含事件记录类型
    """
    for data_type in EventRecordTypes:
        if data_type.data_format.find(",") == -1:
            value = EventRecord(
                data_type.di, pad_with_zeros(len(data_type.data_format))
            )
        else:
            value = EventRecord(
                data_type.di,
                (
                    pad_with_zeros(len(data_type.data_format.split(",")[0])),
                    pad_with_zeros(len(data_type.data_format.split(",")[1])),
                ),
            )

        if data_type.di not in DIMap:
            DIMap[data_type.di] = []
        data_item = DataItem(
            di=data_type.di,
            name=data_type.name,
            data_format=data_type.data_format,
            value=value,
            unit=data_type.unit,
        )
        DIMap[data_type.di].append(data_item)



import json

from src.common import root_path


def fix_dataformat_key(data_list):
    """
    将列表中的Dataformat键改为DataFormat
    """

    fixed_data = []
    for item in data_list:
        # 创建新字典，修改键名
        new_item = item.copy()
        if "Dataformat" in new_item:
            new_item["DataFormat"] = new_item.pop("Dataformat")
        fixed_data.append(new_item)
    return fixed_data


# 执行修改
# 读取数据
with open(root_path + '/config/variable_types.json', 'r', encoding='utf-8') as f:
    data = json.load(f)

fixed_data = fix_dataformat_key(data)

# 输出结果
print("修改后的数据:")
print(json.dumps(fixed_data, ensure_ascii=False, indent=4))

# 如果需要保存到文件
with open(root_path + '/config/variable_types.json', 'w', encoding='utf-8') as f:
    json.dump(fixed_data, f, ensure_ascii=False, indent=4)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
DLT645协议库报文捕获功能测试

这个脚本提供了报文捕获功能的单元测试
"""

import unittest
import sys
import os
import time
import threading

# 添加项目根目录到Python路径
project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, project_root)

try:
    from src.common.message_types import MessageRecord, MessagePair
    from src.common.message_capture import MessageQueue, MessageCapture
except ImportError as e:
    print(f"导入模块失败: {e}")
    print("请确保在dlt645/python目录中运行此脚本")
    sys.exit(1)


class TestMessageTypes(unittest.TestCase):
    """报文数据类型测试"""

    def test_message_record_creation(self):
        """测试MessageRecord创建"""
        print("测试MessageRecord创建...")
        record = MessageRecord(direction="TX", data=b"\x68\x00\x00\x00\x00\x00\x00\x68")
        self.assertEqual(record.direction, "TX")
        self.assertEqual(record.data, b"\x68\x00\x00\x00\x00\x00\x00\x68")
        self.assertIsNotNone(record.id)
        self.assertIsNotNone(record.timestamp)
        self.assertIn("68", record.hex_string)
        print("✓ MessageRecord创建成功")

    def test_message_record_to_dict(self):
        """测试MessageRecord转换为字典"""
        print("测试MessageRecord转换为字典...")
        record = MessageRecord(direction="RX", data=b"\x68\x01\x02\x03\x04\x05\x06\x68")
        record_dict = record.to_dict()
        self.assertIn("id", record_dict)
        self.assertEqual(record_dict["direction"], "RX")
        self.assertIn("timestamp", record_dict)
        print("✓ MessageRecord转换为字典成功")

    def test_message_pair_creation(self):
        """测试MessagePair创建"""
        print("测试MessagePair创建...")
        pair = MessagePair()
        self.assertIsNotNone(pair.id)
        self.assertIsNone(pair.tx)
        self.assertIsNone(pair.rx)
        self.assertFalse(pair.is_complete())
        print("✓ MessagePair创建成功")

    def test_message_pair_with_tx_rx(self):
        """测试MessagePair配对TX和RX"""
        print("测试MessagePair配对TX和RX...")
        pair = MessagePair()
        
        tx_record = MessageRecord(direction="TX", data=b"\x68\x00")
        time.sleep(0.01)  # 小延迟确保时间差
        rx_record = MessageRecord(direction="RX", data=b"\x68\x01")
        
        pair.set_tx(tx_record)
        pair.set_rx(rx_record)
        
        self.assertTrue(pair.is_complete())
        self.assertIsNotNone(pair.round_trip_time)
        self.assertGreater(pair.round_trip_time, 0)
        self.assertEqual(tx_record.pair_id, pair.id)
        self.assertEqual(rx_record.pair_id, pair.id)
        print("✓ MessagePair配对TX和RX成功")


class TestMessageQueue(unittest.TestCase):
    """报文队列测试"""

    def test_queue_basic_operations(self):
        """测试队列基本操作"""
        print("测试队列基本操作...")
        queue = MessageQueue(maxlen=10)
        
        # 添加记录
        record = MessageRecord(direction="TX", data=b"\x68")
        queue.append(record)
        
        self.assertEqual(len(queue), 1)
        records = queue.get_all()
        self.assertEqual(len(records), 1)
        print("✓ 队列基本操作成功")

    def test_queue_maxlen(self):
        """测试队列最大长度限制"""
        print("测试队列最大长度限制...")
        queue = MessageQueue(maxlen=5)
        
        # 添加超过最大长度的记录
        for i in range(10):
            record = MessageRecord(direction="TX", data=bytes([i]))
            queue.append(record)
        
        self.assertEqual(len(queue), 5)
        # 应该保留最后5条记录
        records = queue.get_all()
        self.assertEqual(records[-1].data, bytes([9]))
        print("✓ 队列最大长度限制成功")

    def test_queue_get_recent(self):
        """测试获取最近记录"""
        print("测试获取最近记录...")
        queue = MessageQueue(maxlen=20)
        
        for i in range(10):
            record = MessageRecord(direction="TX", data=bytes([i]))
            queue.append(record)
        
        recent = queue.get_recent(3)
        self.assertEqual(len(recent), 3)
        self.assertEqual(recent[-1].data, bytes([9]))
        print("✓ 获取最近记录成功")

    def test_queue_clear(self):
        """测试清空队列"""
        print("测试清空队列...")
        queue = MessageQueue(maxlen=10)
        
        for i in range(5):
            queue.append(MessageRecord(direction="TX", data=bytes([i])))
        
        queue.clear()
        self.assertEqual(len(queue), 0)
        print("✓ 清空队列成功")

    def test_queue_resize(self):
        """测试调整队列大小"""
        print("测试调整队列大小...")
        queue = MessageQueue(maxlen=10)
        
        for i in range(8):
            queue.append(MessageRecord(direction="TX", data=bytes([i])))
        
        # 缩小队列
        queue.resize(5)
        self.assertEqual(len(queue), 5)
        self.assertEqual(queue.maxlen, 5)
        
        # 确保保留的是最新的记录
        records = queue.get_all()
        self.assertEqual(records[-1].data, bytes([7]))
        print("✓ 调整队列大小成功")

    def test_queue_thread_safety(self):
        """测试队列线程安全性"""
        print("测试队列线程安全性...")
        queue = MessageQueue(maxlen=1000)
        
        def add_records(start, count):
            for i in range(start, start + count):
                queue.append(MessageRecord(direction="TX", data=bytes([i % 256])))
        
        # 创建多个线程同时添加记录
        threads = []
        for i in range(10):
            t = threading.Thread(target=add_records, args=(i * 50, 50))
            threads.append(t)
        
        for t in threads:
            t.start()
        
        for t in threads:
            t.join()
        
        # 应该有500条记录
        self.assertEqual(len(queue), 500)
        print("✓ 队列线程安全性测试成功")


class TestMessageCapture(unittest.TestCase):
    """报文捕获管理器测试"""

    def test_capture_disabled_by_default(self):
        """测试默认禁用捕获"""
        print("测试默认禁用捕获...")
        capture = MessageCapture()
        
        tx_id = capture.capture_tx(b"\x68\x00")
        self.assertIsNone(tx_id)
        
        stats = capture.get_stats()
        self.assertFalse(stats["enabled"])
        print("✓ 默认禁用捕获测试成功")

    def test_capture_enable_disable(self):
        """测试启用和禁用捕获"""
        print("测试启用和禁用捕获...")
        capture = MessageCapture()
        
        capture.enable()
        self.assertTrue(capture.enabled)
        
        capture.disable()
        self.assertFalse(capture.enabled)
        print("✓ 启用/禁用捕获测试成功")

    def test_capture_tx_and_rx(self):
        """测试捕获TX和RX"""
        print("测试捕获TX和RX...")
        capture = MessageCapture(enabled=True, queue_size=50)
        
        # 捕获TX
        tx_id = capture.capture_tx(b"\x68\x00\x00\x00\x00\x00\x00\x68")
        self.assertIsNotNone(tx_id)
        
        # 捕获RX并配对
        capture.capture_rx(b"\x68\x01\x02\x03\x04\x05\x06\x68", tx_id=tx_id)
        
        # 验证捕获结果
        tx_messages = capture.get_tx_messages()
        self.assertEqual(len(tx_messages), 1)
        
        rx_messages = capture.get_rx_messages()
        self.assertEqual(len(rx_messages), 1)
        
        pairs = capture.get_pairs()
        self.assertEqual(len(pairs), 1)
        self.assertTrue(pairs[0].is_complete())
        print("✓ 捕获TX和RX测试成功")

    def test_capture_server_mode(self):
        """测试服务器模式捕获（RX先于TX）"""
        print("测试服务器模式捕获...")
        capture = MessageCapture(enabled=True, queue_size=50)
        
        # 服务器先收到RX（请求）
        rx_id = capture.capture_rx_for_server(b"\x68\x00\x00\x00\x00\x00\x00\x68")
        self.assertIsNotNone(rx_id)
        
        # 然后发送TX（响应）
        capture.capture_tx_for_server(b"\x68\x01\x02\x03\x04\x05\x06\x68", rx_id=rx_id)
        
        # 验证配对
        pairs = capture.get_pairs()
        self.assertEqual(len(pairs), 1)
        self.assertTrue(pairs[0].is_complete())
        print("✓ 服务器模式捕获测试成功")

    def test_capture_queue_size(self):
        """测试队列大小设置"""
        print("测试队列大小设置...")
        capture = MessageCapture(enabled=True, queue_size=5)
        
        # 发送超过队列大小的报文
        for i in range(10):
            capture.capture_tx(bytes([0x68, i]))
        
        tx_messages = capture.get_tx_messages()
        self.assertEqual(len(tx_messages), 5)
        print("✓ 队列大小设置测试成功")

    def test_capture_clear(self):
        """测试清空捕获"""
        print("测试清空捕获...")
        capture = MessageCapture(enabled=True, queue_size=50)
        
        for i in range(5):
            capture.capture_tx(bytes([0x68, i]))
            capture.capture_rx(bytes([0x68, i + 10]))
        
        capture.clear()
        
        self.assertEqual(len(capture.get_tx_messages()), 0)
        self.assertEqual(len(capture.get_rx_messages()), 0)
        print("✓ 清空捕获测试成功")

    def test_capture_stats(self):
        """测试捕获统计"""
        print("测试捕获统计...")
        capture = MessageCapture(enabled=True, queue_size=100)
        
        for i in range(3):
            tx_id = capture.capture_tx(bytes([0x68, i]))
            capture.capture_rx(bytes([0x68, i + 10]), tx_id=tx_id)
        
        stats = capture.get_stats()
        self.assertTrue(stats["enabled"])
        self.assertEqual(stats["queue_size"], 100)
        self.assertEqual(stats["tx_count"], 3)
        self.assertEqual(stats["rx_count"], 3)
        self.assertEqual(stats["pair_count"], 3)
        print("✓ 捕获统计测试成功")


def run_tests():
    """运行测试"""
    print("DLT645协议库报文捕获功能测试")
    print("=" * 50)

    # 创建测试套件
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    
    suite.addTests(loader.loadTestsFromTestCase(TestMessageTypes))
    suite.addTests(loader.loadTestsFromTestCase(TestMessageQueue))
    suite.addTests(loader.loadTestsFromTestCase(TestMessageCapture))

    # 运行测试
    runner = unittest.TextTestRunner(verbosity=2)
    result = runner.run(suite)

    print("\n" + "=" * 50)
    if result.wasSuccessful():
        print("✓ 所有测试通过！")
        print("报文捕获功能正常工作。")
    else:
        print("✗ 部分测试失败！")
        print("请检查错误信息并修复问题。")

    return result.wasSuccessful()


if __name__ == "__main__":
    success = run_tests()
    sys.exit(0 if success else 1)



"""DLT645协议Python实现库

这个库提供了DLT645通信协议的完整实现，包括：
- TCP和RTU通信方式
- 客户端和服务端功能
- 电能表数据读写操作
- 电能量、最大需量、变量数据类型支持
- 实时报文捕获功能
"""

__version__ = "1.0.0"
__author__ = "Chen Dongyu"
__email__ = "1755696012@qq.com"

# 导入主要的服务类
from .service.serversvc.server_service import (
    MeterServerService,
)
from .service.clientsvc.client_service import MeterClientService

# 导入协议相关
from .protocol.protocol import DLT645Protocol
from .model.types.dlt645_type import CtrlCode, Demand
from .model.types.data_type import DataItem, DataFormat

# 导入传输层
from .transport.server.tcp_server import TcpServer
from .transport.server.rtu_server import RtuServer
from .transport.client.tcp_client import TcpClient
from .transport.client.rtu_client import RtuClient

# 导入报文捕获模块
from .common.message_types import MessageRecord, MessagePair
from .common.message_capture import MessageCapture, MessageQueue

# 导出所有公共接口
__all__ = [
    # 版本信息
    "__version__",
    "__author__",
    "__email__",
    
    # 服务类
    "MeterServerService",
    "MeterClientService",
    
    # 协议类
    "DLT645Protocol",
    
    # 数据类型
    "CtrlCode",
    "Demand",
    "DataItem",
    "DataFormat",
    
    # 传输层
    "TcpServer",
    "RtuServer",
    "TcpClient",
    "RtuClient",
    
    # 报文捕获
    "MessageRecord",
    "MessagePair",
    "MessageCapture",
    "MessageQueue",
]


"""TCP 服务器模块。

本模块实现了 DLT645 协议的 TCP 服务器功能。
"""

import socket
import threading
import time
from typing import Optional

from ...common.transform import bytes_to_spaced_hex
from ...common.message_capture import MessageCapture
from ...protocol.protocol import DLT645Protocol
from ...transport.server.log import log


class TcpServer:
    """TCP 服务器类，用于与 DLT645 客户端进行 TCP 通信。

    :ivar ip: 服务器 IP 地址。
    :ivar port: 服务器端口号。
    :ivar timeout: 连接超时时间（秒）。
    :ivar ln: 监听套接字。
    :ivar service: 服务实例，用于处理业务逻辑。
    """

    def __init__(self, ip: str, port: int, timeout: float, service):
        """初始化 TCP 服务器。

        :param ip: 服务器 IP 地址（如 '0.0.0.0'）。
        :type ip: str
        :param port: 服务器端口号。
        :type port: int
        :param timeout: 连接超时时间（秒）。
        :type timeout: float
        :param service: 服务实例，用于处理业务逻辑。
        :type service: Any
        """
        self.ip = ip
        self.port = port
        self.timeout = timeout
        self.ln = None
        self.service = service
        self._server_thread = None
        self._running = False
        self._stop_event = threading.Event()
        # 跟踪所有活跃的客户端连接
        self._connections = []
        # 用于保护连接列表的锁
        self._connections_lock = threading.Lock()
        # 报文捕获管理器
        self._message_capture: Optional[MessageCapture] = None

    def start(self):
        """启动TCP服务器（非阻塞，在后台线程中运行）"""
        if self._running:
            log.warning("TCP server is already running")
            return None

        self._stop_event.clear()
        self._running = True

        # 在后台线程中启动服务器
        self._server_thread = threading.Thread(target=self._run_server, daemon=True)
        self._server_thread.start()

        # 等待服务器启动完成
        time.sleep(0.1)
        log.info(f"TCP server starting in background on {self.ip}:{self.port}")
        return None

    def _run_server(self):
        """服务器主循环（在后台线程中运行）"""
        try:
            # 创建 TCP 套接字
            self.ln = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # 设置地址可重用
            self.ln.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            # 设置非阻塞超时，以便能够响应停止信号
            self.ln.settimeout(1.0)
            # 绑定地址和端口
            self.ln.bind((self.ip, self.port))
            # 开始监听
            self.ln.listen(5)
            log.info(f"TCP server started on port {self.port}")

            while not self._stop_event.is_set():
                try:
                    # 接受连接
                    conn, addr = self.ln.accept()
                    log.info(f"Accepted connection from {addr}")
                    # 设置超时时间
                    conn.settimeout(self.timeout)

                    # 将连接添加到活跃连接列表
                    with self._connections_lock:
                        self._connections.append(conn)

                    # 启动新线程处理连接
                    threading.Thread(
                        target=self.handle_connection, args=(conn,), daemon=True
                    ).start()
                except socket.timeout:
                    # 超时是正常的，继续检查停止信号
                    continue
                except socket.error as e:
                    if self._stop_event.is_set():
                        break
                    log.error(f"Failed to accept connection: {e}")
                    if hasattr(e, "errno") and e.errno == 10038:  # 套接字关闭错误
                        break
        except Exception as e:
            log.error(f"Failed to start TCP server: {e}")
        finally:
            self._running = False
            if self.ln:
                try:
                    self.ln.close()
                except:
                    pass
            log.info("TCP server stopped")

    def stop(self):
        """停止TCP服务器"""
        if not self._running:
            log.warning("TCP server is not running")
            return None

        log.info("Shutting down TCP server...")

        # 设置停止信号
        self._stop_event.set()

        # 主动关闭所有活跃的客户端连接
        with self._connections_lock:
            for conn in self._connections:
                try:
                    conn.close()
                    log.info(f"Closed active connection: {conn}")
                except Exception as e:
                    log.error(f"Error closing client connection: {e}")
            # 清空连接列表
            self._connections.clear()

        # 关闭服务器套接字
        if self.ln:
            try:
                self.ln.close()
            except Exception as e:
                log.error(f"Error closing server socket: {e}")

        # 等待服务器线程结束
        if self._server_thread and self._server_thread.is_alive():
            self._server_thread.join(timeout=5.0)
            if self._server_thread.is_alive():
                log.warning("Server thread did not stop gracefully")

        self._running = False
        log.info("TCP server shutdown complete")
        return None

    def is_running(self):
        """检查服务器是否正在运行"""
        return self._running

    def handle_connection(self, conn):
        try:
            # 初始化数据缓冲区，用于累积接收的数据
            data_buffer = bytearray()

            while not self._stop_event.is_set():
                try:
                    # 接收数据
                    buf = conn.recv(256)

                    if not buf:
                        continue

                    # 将新接收的数据添加到缓冲区
                    data_buffer.extend(buf)
                    log.info(f"RX: {bytes_to_spaced_hex(buf)} (len:{len(data_buffer)})")
                    
                    # 捕获接收的报文
                    current_tx_id: Optional[str] = None
                    if self._message_capture:
                        current_tx_id = self._message_capture.capture_rx_for_server(buf)

                    # 尝试解析缓冲区中的数据
                    try:
                        # 尝试从缓冲区中提取完整的帧
                        remaining_data, frame = (
                            DLT645Protocol.deserialize_with_remaining(
                                bytes(data_buffer)
                            )
                        )

                        if frame is not None:
                            # 解析成功，更新缓冲区为剩余未解析的数据
                            data_buffer = bytearray(remaining_data)
                            log.info(
                                f"Successfully parsed frame, remaining buffer size: {len(remaining_data)}"
                            )

                            # 业务处理
                            try:
                                resp = self.service.handle_request(frame)

                                # 响应
                                if resp:
                                    try:
                                        conn.sendall(resp)
                                        log.info(
                                            f"TX: {bytes_to_spaced_hex(resp)} (len:{len(resp)})"
                                        )
                                        # 捕获发送的报文并与接收配对
                                        if self._message_capture:
                                            self._message_capture.capture_tx_for_server(resp, current_tx_id)
                                    except Exception as e:
                                        log.error(f"Error writing response: {e}")
                            except Exception as e:
                                log.error(f"Error handling request: {e}")

                            # 继续循环，处理缓冲区中剩余的数据
                            continue

                        # 如果没有完整的帧，但收到了新数据，继续等待
                        # 注意：这里不会清空缓冲区，而是保持数据以等待后续数据

                    except Exception as e:
                        # 解析错误，但不立即清空缓冲区
                        # 可能是因为数据不完整，等待更多数据
                        log.warning(
                            f"Error parsing frame (might be incomplete data): {e}"
                        )
                        # 只有在确定是无效数据格式时才清空缓冲区
                        # 这里可以根据具体的异常类型或错误信息来判断

                except socket.timeout:
                    # 清空缓冲区
                    if len(data_buffer) > 0:
                        log.warning(
                            f"Connection timeout with {len(data_buffer)} bytes in buffer, clearing buffer {bytes_to_spaced_hex(data_buffer)}"
                        )
                    data_buffer.clear()
        except Exception as e:
            log.error(f"Error handling connection: {e}")
        finally:
            try:
                # 将连接从活跃连接列表中移除
                with self._connections_lock:
                    if conn in self._connections:
                        self._connections.remove(conn)

                conn.close()
                log.info("Connection closed")
            except Exception as e:
                log.error(f"Error closing connection: {e}")



"""数据项处理模块。

本模块提供了数据项的获取和设置功能，用于管理 DLT645 协议中各类数据项的值。
"""

from typing import Optional, Union, List

from ...model.types.data_type import DataItem, DataFormat
from ...model.types.dlt645_type import Demand, EventRecord
from ...model.log import log
from .define import DIMap


def get_data_item(di: int) -> Optional[DataItem | List[DataItem]]:
    """根据数据标识 (DI) 获取数据项。

    :param di: 数据标识，4字节整数。
    :type di: int
    :return: 对应的数据项，可能是单个 DataItem 或 DataItem 列表。
             如果未找到则返回 None。
    :rtype: Optional[DataItem | List[DataItem]]
    """
    item = DIMap.get(di)
    if item is None:
        log.error(f"未通过di {hex(di)} 找到映射")
        return None
    return item


def set_data_item(di: int, data: Union[int, float, str, Demand, list, tuple]) -> bool:
    """设置指定数据标识 (DI) 的数据项值。

    根据 DI 的类型自动处理不同的数据格式：
    - 需量数据 (Demand): 验证值后直接设置
    - 事件记录 (0x03xxxxxx): 批量设置事件记录值
    - 参变量时段表 (0x04xxxxxx): 批量设置时段表值
    - 其他数据: 验证后直接设置

    :param di: 数据标识，4字节整数。
    :type di: int
    :param data: 要设置的数据值，类型取决于数据项类型。
    :type data: Union[int, float, str, Demand, list, tuple]
    :return: 设置成功返回 True，失败返回 False。
    :rtype: bool
    """
    if di in DIMap:
        item = DIMap[di]
        if isinstance(data, Demand):
            if not is_value_valid(item.data_format, data.value):
                log.error(f"值 {data} 不符合数据格式: {item.data_format}")
                return False
            item.value = data
        elif 0x03010000 <= di <= 0x03300E0A:  # 事件记录数据
            for data_item, value in zip(item, data):  # data的每一条数据是一个事件记录
                if not is_value_valid(data_item.data_format, value):
                    log.error(f"值 {value} 不符合数据格式: {data_item.data_format}")
                    return False
                data_item.value.event = value
        elif 0x04010000 <= di <= 0x04020008:  # 参变量时段表数据
            for data_item, value in zip(item, data):
                if not is_value_valid(data_item.data_format, value):
                    log.error(f"值 {value} 不符合数据格式: {data_item.data_format}")
                    return False
                data_item.value = value
        else:
            if not is_value_valid(item.data_format, data):
                log.error(f"值 {data} 不符合数据格式: {item.data_format}")
                return False
            item.value = data
        log.debug(f"设置数据项 {hex(di)} 成功, 值 {item}")
        return True
    return False


def is_value_valid(data_format: str, value: Union[int, float, str, tuple]) -> bool:
    """检查值是否符合指定的数据格式。

    根据数据格式字符串验证值的有效范围：
    - XXXXXX.XX: 范围 [-799999.99, 799999.99]
    - XXXX.XX: 范围 [-7999.99, 7999.99]
    - XXX.XXX: 范围 [-799.999, 799.999]
    - XX.XXXX: 范围 [-79.9999, 79.9999]
    - XXX.X: 范围 [-799.9, 799.9]
    - X.XXX: 范围 [-0.999, 0.999]
    - 其他格式: 检查字符串长度或递归验证元组

    :param data_format: 数据格式字符串。
    :type data_format: str
    :param value: 待验证的值。
    :type value: Union[int, float, str, tuple]
    :return: 值有效返回 True，无效返回 False。
    :rtype: bool
    """
    if data_format == DataFormat.XXXXXX_XX.value:
        return -799999.99 <= value <= 799999.99
    elif data_format == DataFormat.XXXX_XX.value:
        return -7999.99 <= value <= 7999.99
    elif data_format == DataFormat.XXX_XXX.value:
        return -799.999 <= value <= 799.999
    elif data_format == DataFormat.XX_XXXX.value:
        return -79.9999 <= value <= 79.9999
    elif data_format == DataFormat.XXX_X.value:
        return -799.9 <= value <= 799.9
    elif data_format == DataFormat.X_XXX.value:
        return -0.999 <= value <= 0.999
    else:
        if isinstance(value, str) and len(value) == len(data_format):
            return True
        elif isinstance(value, tuple):
            fmt = data_format.split(",")
            for v, fmt in zip(value, fmt):
                if not is_value_valid(fmt, v):
                    return False
            return True
        else:
            return False




from ast import List
from enum import Enum
from typing import Union
import datetime
import json
from ...model.log import log


class DataItem:
    """数据项实体类，表示一个具体的数据项。

    :ivar di: 数据标识 (DI)，4 字节。
    :ivar name: 数据项名称。
    :ivar data_format: 数据格式字符串（如 "XXXXXX.XX"）。
    :ivar value: 数据值。
    :ivar unit: 数据单位。
    :ivar update_time: 最后更新时间戳。
    """

    def __init__(
        self,
        di: int,
        name: str,
        data_format: str,
        value: Union[str, float, List] = 0,
        unit: str = "",
        update_time: datetime.datetime = datetime.datetime.now(),
    ):
        """初始化 DataItem 实例。

        :param di: 数据标识 (DI)。
        :type di: int
        :param name: 数据项名称。
        :type name: str
        :param data_format: 定义数据结构的格式。
        :type data_format: str
        :param value: 数据项的实际值，默认为 0。
        :type value: Union[str, float, List], 可选
        :param unit: 单位字符串（如 "kWh"），默认为空字符串。
        :type unit: str, 可选
        :param update_time: 数据更新的时间戳，默认为当前时间。
        :type update_time: datetime.datetime, 可选
        """
        self.di = di
        self.name = name
        self.data_format = data_format
        self.value = value
        self.unit = unit
        self.update_time = update_time

    def __repr__(self):
        """返回 DataItem 的字符串表示。

        :return: 字符串表示。
        :rtype: str
        """
        return (
            f"DataItem(name={self.name}, di={format(self.di, '#x')}, value={self.value}, "
            f"unit={self.unit},data_format={self.data_format}, timestamp={datetime.datetime.strftime(self.update_time, '%Y-%m-%d %H:%M:%S')})"
        )


class DataType:
    """数据类型配置类，通常从 JSON 加载。

    :ivar di: 数据标识 (DI)，整数形式。
    :ivar name: 数据类型名称。
    :ivar unit: 计量单位。
    :ivar data_format: 格式字符串定义。
    """

    def __init__(self, Di="", Name="", Unit="", DataFormat=""):
        """初始化 DataType 实例。

        :param Di: 数据标识字符串（十六进制），默认为空字符串。
        :type Di: str, 可选
        :param Name: 数据类型名称，默认为空字符串。
        :type Name: str, 可选
        :param Unit: 计量单位，默认为空字符串。
        :type Unit: str, 可选
        :param DataFormat: 格式字符串，默认为空字符串。
        :type DataFormat: str, 可选
        """
        self.di = uint32_from_string.from_json(Di)
        self.name = Name
        self.unit = Unit
        self.data_format = DataFormat

    @classmethod
    def from_dict(cls, data):
        """从字典创建 DataType 实例。

        :param data: 包含 'Di', 'Name', 'Unit', 'DataFormat' 的字典。
        :type data: dict
        :return: DataType 实例。
        :rtype: DataType
        """
        return cls(**data)


class uint32_from_string(int):
    """自定义整数类型，用于处理从 JSON 字符串（包括十六进制）解析 uint32。"""

    @classmethod
    def from_json(cls, data):
        """将输入数据解析为 uint32 整数。

        处理以下情况：
        - 空字符串 -> 0
        - 十六进制字符串（带或不带 0x 前缀）
        - 普通整数

        :param data: 待解析的输入数据。
        :type data: Union[str, int]
        :return: 解析后的整数值。
        :rtype: uint32_from_string
        :raises ValueError: 如果转换失败。
        """
        if data == "":
            return cls(0)
        if isinstance(data, str):
            try:
                return cls(int(data, 16))
            except ValueError as e:
                raise ValueError(f"无法转换为 uint32: {e}")
        return cls(data)


def init_data_type_from_json(file_path: str):
    """从 JSON 文件初始化 DataType 对象列表。

    读取指定的 JSON 文件，解析后返回 DataType 实例列表。

    :param file_path: JSON 配置文件的路径。
    :type file_path: str
    :return: 初始化后的 DataType 对象列表。
    :rtype: list[DataType]
    :raises FileNotFoundError: 如果 JSON 文件不存在。
    :raises json.JSONDecodeError: 如果 JSON 文件格式无效。
    """
    try:
        # 读取 JSON 文件
        with open(file_path, "r", encoding="utf-8") as f:
            json_data = json.load(f)

        # 解析 JSON 到列表
        data_types = [DataType.from_dict(item) for item in json_data]
        # log.debug(f"初始化 {file_path} 完成，共加载 {len(data_types)} 种数据类型")
        return data_types
    except FileNotFoundError as e:
        log.error(f"读取文件失败: {e}")
        raise
    except json.JSONDecodeError as e:
        log.error(f"解析 JSON 失败: {e}")
        raise


class DataFormat(Enum):
    """表示各种数据格式模板的枚举。

    定义说明：
    - X: 十进制数字
    - Y: 年
    - M: 月
    - D: 日
    - W: 星期
    - h: 时
    - m: 分
    - s: 秒
    - N: 数字（通用）
    """

    XXXXXXXX = "XXXXXXXX"
    XXXXXX_XX = "XXXXXX.XX"
    XXXX_XX = "XXXX.XX"
    XXX_XXX = "XXX.XXX"
    XX_XXXX = "XX.XXXX"
    XXX_X = "XXX.X"
    X_XXX = "X.XXX"
    YYMMDDWW = "YYMMDDWW"  # 日年月日星期
    hhmmss = "hhmmss"  # 时分秒
    YYMMDDhhmm = "YYMMDDhhmm"  # 日年月日时分
    NN = "NN"
    NNNN = "NNNN"
    NNNNNNNN = "NNNNNNNN"



from typing import List
from datetime import datetime

from ....model.types.data_type import DataItem, DataFormat
from ....model.types.dlt645_type import Demand
from . import DIMap

# 需量DI列表
demand_di_list = [
    0x01150000,
    0x01160000,
    0x01170000,
    0x01180000,
    0x01190000,
    0x011A0000,
    0x011B0000,
    0x011C0000,
    0x011D0000,
    0x011E0000,
    0x01290000,
    0x012A0000,
    0x012B0000,
    0x012C0000,
    0x012D0000,
    0x012E0000,
    0x012F0000,
    0x01300000,
    0x01310000,
    0x01320000,
    0x013D0000,
    0x013E0000,
    0x013F0000,
    0x01400000,
    0x01410000,
    0x01420000,
    0x01430000,
    0x01440000,
    0x01450000,
    0x01460000,
]


def init_demand_def(demand_types: List[DataItem]):
    """初始化需求定义
    
    Args:
        demand_types: 需求类型列表，包含正向有功、反向有功等类型
    """
    di3 = 0  # 数据类型
    di2 = 0  # 电能类型
    di1 = 0  # 同一类型电能里的不同项
    di0 = 0  # 结算日

    for i in range(64):
        for j in range(13):
            if j == 0:
                name_prefix = "（当前）"
            else:
                name_prefix = f"（上{j}结算日）"

            # 正向有功需量
            key = (di3 + 1) << 24 | (di2 + 1) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 反向有功需量
            key = (di3 + 1) << 24 | (di2 + 2) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 组合无功1需量
            key = (di3 + 1) << 24 | (di2 + 3) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 * 2 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 * 2 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 组合无功2需量
            key = (di3 + 1) << 24 | (di2 + 4) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 * 3 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 * 3 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 第一象限无功费率最大需量
            key = (di3 + 1) << 24 | (di2 + 5) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 * 4 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 * 4 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 第二象限无功费率最大需量
            key = (di3 + 1) << 24 | (di2 + 6) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 * 5 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 * 5 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 第三象限无功费率最大需量
            key = (di3 + 1) << 24 | (di2 + 7) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 * 6 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 * 6 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 第四象限无功费率最大需量
            key = (di3 + 1) << 24 | (di2 + 8) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 * 7 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 * 7 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 正向视在最大需量
            key = (di3 + 1) << 24 | (di2 + 9) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 * 8 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 * 8 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 反向视在最大需量
            key = (di3 + 1) << 24 | (di2 + 10) << 16 | (di1 + i) << 8 | (di0 + j)
            DIMap[key] = DataItem(
                di=key,
                name=name_prefix + demand_types[64 * 9 + i].name,
                data_format=DataFormat.XX_XXXX.value,
                unit=demand_types[64 * 9 + i].unit,
                value=Demand(0.0, datetime.now()),
            )

            # 最后几个数据特殊处理
            for k in range(len(demand_di_list)):
                # 提取demandDiList中的前24位，然后添加结算日信息（最后8位）
                key = (demand_di_list[k] & 0xFFFFFF00) | (di0 + j)
                DIMap[key] = DataItem(
                    di=key,
                    name=name_prefix + demand_types[64 * 10 + k].name,
                    data_format=DataFormat.XX_XXXX.value,
                    unit=demand_types[64 * 10 + k].unit,
                    value=Demand(0.0, datetime.now()),
                )



"""RTU 客户端模块。

本模块实现了 DLT645 协议的 RTU（串口）客户端功能。
"""

import time
import serial
from typing import Optional
from ...common.transform import bytes_to_spaced_hex
from ...common.message_capture import MessageCapture
from ...transport.client.log import log
from ...protocol.frame import FRAME_START_BYTE, FRAME_END_BYTE
from ...protocol.protocol import DLT645Protocol


class RtuClient:
    """RTU 客户端类，用于与 DLT645 设备进行串口通信。

    该类实现了 RTU（Remote Terminal Unit）客户端功能，
    支持与 DLT645 协议设备进行串口通信。

    :ivar port: 串口名称。
    :ivar baud_rate: 波特率。
    :ivar data_bits: 数据位。
    :ivar stop_bits: 停止位。
    :ivar parity: 校验位。
    :ivar timeout: 超时时间（秒）。
    :ivar conn: 串口连接对象。
    """

    def __init__(
        self,
        port: str = "",
        baud_rate: int = 9600,
        data_bits: int = 8,
        stop_bits: int = 1,
        parity: str = serial.PARITY_NONE,
        timeout: float = 1.0,
    ):
        """初始化 RTU 客户端。

        :param port: 串口名称（如 "COM1" 或 "/dev/ttyUSB0"）。
        :type port: str
        :param baud_rate: 波特率，默认 9600。
        :type baud_rate: int
        :param data_bits: 数据位，默认 8。
        :type data_bits: int
        :param stop_bits: 停止位，默认 1。
        :type stop_bits: int
        :param parity: 校验位，默认无校验。
        :type parity: str
        :param timeout: 超时时间（秒），默认 1.0。
        :type timeout: float
        """
        self.port = port
        self.baud_rate = baud_rate
        self.data_bits = data_bits
        self.stop_bits = stop_bits
        self.parity = parity
        self.timeout = timeout
        self.conn: Optional[serial.Serial] = None
        # 报文捕获管理器
        self._message_capture: Optional[MessageCapture] = None

    def connect(self) -> bool:
        """连接到串口
        
        建立与指定串口的连接，配置为DLT645协议通信所需的参数。
        
        Returns:
            bool: 连接成功返回True，失败返回False
        """
        try:
            self.conn = serial.Serial(
                port=self.port,
                baudrate=self.baud_rate,
                bytesize=self.data_bits,
                stopbits=self.stop_bits,
                parity=self.parity,
                timeout=self.timeout,
            )
            log.info(f"RTU client connected to port {self.port}")
            return True
        except Exception as e:
            log.error(f"Failed to open serial port: {e}")
            return False

    def _is_valid_response(self, response: bytearray) -> bool:
        """检查响应是否为有效的DLT645帧

        Args:
            response: 接收到的响应数据

        Returns:
            bool: 如果响应包含完整的DLT645帧则返回True
        """
        # 检查是否同时包含起始字节和结束字节
        if FRAME_START_BYTE in response and FRAME_END_BYTE in response:
            # 确保结束字节在起始字节之后
            start_pos = response.find(FRAME_START_BYTE)
            end_pos = response.find(FRAME_END_BYTE, start_pos)
            if end_pos > start_pos:
                # 检查帧长度是否合理（最小帧长度约为12字节）
                if end_pos - start_pos >= 12:
                    return True
        return False

    def disconnect(self) -> bool:
        """断开与串口的连接
        
        关闭串口连接并释放资源。
        
        Returns:
            bool: 断开连接成功返回True，失败返回False
        """
        if self.conn is not None:
            try:
                self.conn.close()
                self.conn = None
                log.info(f"RTU client disconnected from port {self.port}")
                return True
            except Exception as e:
                log.error(f"Failed to close serial port: {e}")
                return False
        return True

    def _check_timeout(self, start_time: float) -> bool:
        """检查是否超时

        Args:
            start_time: 开始时间

        Returns:
            bool: 如果超时则返回True，否则返回False
        """
        return time.time() - start_time > self.timeout

    def send_request(
        self,
        data: bytes,
        retries: int = 1,
    ) -> Optional[bytes]:
        """发送请求并接收响应
        
        向DLT645设备发送请求数据，并接收响应。支持超时处理和自动重试机制。
        
        Args:
            data: 要发送的请求数据
            retries: 失败重试次数，默认1次

        Returns:
            bytes: 成功接收的响应数据
            None: 失败时返回
        """
        # 确保连接已建立
        if not self._ensure_connection():
            log.error("Failed to establish serial port connection")
            return None

        # 保存原始超时设置
        original_timeout = self.conn.timeout

        for attempt in range(retries + 1):
            last_data_time = time.time()
            try:
                # 清空缓冲区
                if not self._safe_clear_buffer():
                    log.warning("Buffer clearance failed, proceeding anyway")

                # 发送数据
                written = self.conn.write(data)
                if written != len(data):
                    log.error(f"TX: Write incomplete ({written}/{len(data)} bytes)")
                    continue

                log.info(f"TX: {bytes_to_spaced_hex(data)}")

                # 捕获发送的报文
                current_tx_id: Optional[str] = None
                if self._message_capture:
                    current_tx_id = self._message_capture.capture_tx(data)

                # 初始化数据缓冲区和接收状态
                data_buffer = bytearray()
                max_buffer_size = 1024  # 增加缓冲区大小以处理较大的数据

                # 持续读取直到收到完整帧或超时
                while not self._check_timeout(last_data_time):
                    # 读取数据, 在没有数据时等待直到超时
                    chunk = self.conn.read(256)

                    if chunk:
                        data_buffer.extend(chunk)
                        last_data_time = time.time()
                        log.info(f"RX: {bytes_to_spaced_hex(chunk)} len{len(chunk)}")

                    # 无论是否收到数据，只要缓冲区不为空就尝试解析
                    if data_buffer:
                        try:
                            # 使用deserialize_with_remaining解析分片数据
                            remaining_data, frame = (
                                DLT645Protocol.deserialize_with_remaining(data_buffer)
                            )

                            if frame is not None:
                                # 成功解析到完整帧
                                log.info(
                                    f"RX: Successfully parsed frame, buffer size after parsing: {len(remaining_data)}"
                                )
                                # 捕获接收的报文并与发送配对
                                if self._message_capture:
                                    self._message_capture.capture_rx(bytes(data_buffer), current_tx_id)
                                return bytes(data_buffer)
                            else:
                                # 更新缓冲区为未解析的剩余数据
                                data_buffer = remaining_data

                                # 检查缓冲区大小限制
                                if len(data_buffer) > max_buffer_size:
                                    log.warning(
                                        f"RX: Buffer overflow, clearing {len(data_buffer)} bytes"
                                    )
                                    data_buffer.clear()
                        except Exception as e:
                            log.error(
                                f"Error parsing data: {type(e).__name__}: {str(e)}"
                            )
                            # 出错时保留数据继续尝试

                    # 小延迟避免CPU占用过高
                    time.sleep(0.001)

                log.warning(
                    f"RX: Timeout after {original_timeout} seconds, received incomplete data"
                )

            except Exception as e:
                log.error(f"Attempt {attempt} failed: {type(e).__name__}: {str(e)}")

            # 非最后一次尝试时延迟重试
            if attempt < retries:
                log.info(f"Retrying ({attempt + 1}/{retries})...")
                time.sleep(0.5 * (attempt + 1))  # 指数退避

        log.error("All attempts failed")
        return None

    def _safe_clear_buffer(self) -> bool:
        """安全清空串口缓冲区
        
        清空输入和输出缓冲区，避免旧数据干扰新的通信。
        
        Returns:
            bool: 清空成功返回True，失败返回False
        """
        try:
            if self.conn is not None:
                self.conn.reset_input_buffer()
                if hasattr(self.conn, "reset_output_buffer"):
                    self.conn.reset_output_buffer()
                return True
        except Exception as e:
            log.warning(f"Clear buffer failed: {str(e)}")
        return False

    def _ensure_connection(self) -> bool:
        """确保串口连接已建立，如果连接断开则尝试重新连接

        Returns:
            bool: 连接是否成功建立
        """
        try:
            # 检查连接是否存在且打开
            if self.conn is None or not self.conn.is_open:
                log.info(
                    "Connection lost or not established, attempting to reconnect..."
                )
                return self.connect()
            return True
        except Exception as e:
            log.error(f"Connection check failed: {str(e)}")
            # 尝试重新连接
            return self.connect()



import inspect
import json
from loguru import logger
import os
import sys
from typing import Optional, Union

LOG_COLORS = {
    "DEBUG": "\033[1;36m",  # CYAN
    "INFO": "\033[1;32m",  # GREEN
    "WARNING": "\033[1;33m",  # YELLOW
    "ERROR": "\033[1;31m",  # RED
    "CRITICAL": "\033[1;31m",  # RED
    "EXCEPTION": "\033[1;31m",  # RED
}
COLOR_RESET = "\033[1;0m"

class Log:
    # 静态标记：确保只移除一次默认 handler
    _default_handler_removed = False

    def __init__(
        self,
        filename: Optional[str] = None,
        cmdlevel: str = "DEBUG",
        filelevel: str = "INFO",
        backup_count: int = 7,  # 默认保留7天/7个文件
        limit: Union[int, str] = "20 MB",  # 支持字符串格式
        when: Optional[str] = None,
        colorful: bool = True,
        compression: Optional[str] = None,  # 新增压缩功能
        is_backtrace: bool = True,
    ):
        # 只移除 loguru 默认的 stderr handler (ID 0)，避免删除其他模块已添加的 handlers
        if not Log._default_handler_removed:
            try:
                logger.remove(0)  # 只移除默认 handler
            except ValueError:
                pass  # 默认 handler 可能已被移除
            Log._default_handler_removed = True

        self.is_backtrace = is_backtrace
        self.logger = logger.bind(task=filename)
        # 设置日志文件路径
        if filename is None:
            filename = getattr(sys.modules["__main__"], "__file__", "log.py")
            filename = os.path.basename(filename.replace(".py", ".log"))

        # 确保日志目录存在
        log_dir = os.path.abspath(os.path.dirname(filename))
        if not os.path.exists(log_dir):
            os.makedirs(log_dir)

        # 控制台输出配置
        self.logger.add(
            sys.stderr,
            level=cmdlevel,
            format=self._formatter,
            colorize=colorful,
            backtrace=True,
            enqueue=True,
            filter=lambda record: record["extra"].get("task") == filename,
        )

        # 文件输出配置
        rotation_config = self._get_rotation_config(when, limit)
        self.logger.add(
            filename,
            level=filelevel,
            format=self._formatter,
            backtrace=True,
            rotation=rotation_config,
            retention=f"{backup_count} days",
            compression=compression,
            enqueue=True,
            filter=lambda record: record["extra"].get("task") == filename,
        )

    def _formatter(self, record):
        # 处理消息内容
        message = str(record["message"]).replace("{", "{{").replace("}", "}}")
        if isinstance(message, (dict, list)):
            try:
                message = json.dumps(message, ensure_ascii=False)
            except TypeError:
                message = str(message)

        # 处理调用栈和颜色
        if self.is_backtrace:
            frame = inspect.currentframe()
            while frame:
                if (
                    "loguru" not in frame.f_code.co_filename
                    and "base_log.py" not in frame.f_code.co_filename
                ):
                    break
                frame = frame.f_back
            file_info = (
                f"[{os.path.basename(frame.f_code.co_filename)}:{frame.f_code.co_name}:{frame.f_lineno}]"
                if frame
                else f"[{record['file']}:{record['function']}:{record['line']}]"
            )
        else:
            file_info = f"[{record['file']}:{record['line']}]"

        level_color = LOG_COLORS.get(record["level"].name, "")
        return (
            f"{level_color}[{record['time'].strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]}] "
            + file_info
            + f"[{record['level']}] {message}{COLOR_RESET}\n"
        )

    def _get_rotation_config(self, when: Optional[str], limit: Union[int, str]):
        if when:  # 时间轮转
            return when  # "D"（天）、"H"（小时）、"midnight"等
        else:  # 大小轮转
            if isinstance(limit, int):
                return f"{limit / 1024 / 1024} MB"
            return limit  # 直接支持"10 MB"、"1 GB"等字符串格式

    @staticmethod
    def set_logger(**kwargs) -> bool:
        """For backward compatibility."""
        return True

    def debug(self, *args, **kwargs):
        self.logger.debug(*args, **kwargs)

    def info(self, *args, **kwargs):
        self.logger.info(*args, **kwargs)

    def warning(self, *args, **kwargs):
        self.logger.warning(*args, **kwargs)

    def error(self, *args, **kwargs):
        self.logger.error(*args, **kwargs)

    def critical(self, *args, **kwargs):
        self.logger.critical(*args, **kwargs)

    def set_config(
        self,
        filename: Optional[str] = None,
        cmdlevel: str = "DEBUG",
        filelevel: str = "INFO",
        backup_count: int = 7,
        limit: Union[int, str] = "20 MB",
        when: Optional[str] = None,
        colorful: bool = True,
        compression: Optional[str] = None,
    ):
        """动态修改日志配置"""
        # 移除旧的handler
        self.logger.remove()
        
        # 重新添加handler
        if filename is None:
            filename = getattr(sys.modules["__main__"], "__file__", "log.py")
            filename = os.path.basename(filename.replace(".py", ".log"))

        # 确保日志目录存在
        log_dir = os.path.abspath(os.path.dirname(filename))
        if not os.path.exists(log_dir):
            os.makedirs(log_dir)

        # 控制台输出配置
        self.logger.add(
            sys.stderr,
            level=cmdlevel,
            format=self._formatter,
            colorize=colorful,
            backtrace=True,
            enqueue=True,
            filter=lambda record: record["extra"].get("task") == filename,
        )

        # 文件输出配置
        rotation_config = self._get_rotation_config(when, limit)
        self.logger.add(
            filename,
            level=filelevel,
            format=self._formatter,
            backtrace=True,
            rotation=rotation_config,
            retention=f"{backup_count} days",
            compression=compression,
            enqueue=True,
            filter=lambda record: record["extra"].get("task") == filename,
        )

    def exception(self, *args, **kwargs):
        self.logger.exception(*args, **kwargs)


"""TCP 客户端模块。

本模块实现了 DLT645 协议的 TCP 客户端功能。
"""

import socket
import time
from typing import Optional, Any

from ...common.transform import bytes_to_spaced_hex
from ...common.message_capture import MessageCapture
from ...protocol.protocol import DLT645Protocol
from ...transport.client.log import log


class TcpClient:
    """TCP 客户端类，用于与 DLT645 设备进行 TCP 通信。

    :ivar ip: 服务器 IP 地址。
    :ivar port: 服务器端口号。
    :ivar timeout: 连接超时时间（秒）。
    :ivar conn: socket 连接对象。
    """

    def __init__(self, ip: str = "", port: int = 0, timeout: float = 5.0):
        """初始化 TCP 客户端。

        :param ip: 服务器 IP 地址（如 '0.0.0.0'）。
        :type ip: str
        :param port: 服务器端口号。
        :type port: int
        :param timeout: 连接超时时间（秒），默认 5.0。
        :type timeout: float
        """
        self.ip = ip
        self.port = port
        self.timeout = timeout
        self.conn: Optional[socket.socket] = None
        # 报文捕获管理器
        self._message_capture: Optional[MessageCapture] = None

    def connect(self) -> bool:
        """连接到服务器"""
        address = f"{self.ip}:{self.port}"
        try:
            self.conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.conn.settimeout(self.timeout)
            self.conn.connect((self.ip, self.port))
            log.info(f"成功连接到服务器 {address}")
            return True
        except Exception as e:
            log.error(f"连接服务器失败: {e}")
            return False

    def disconnect(self) -> bool:
        """断开与服务器的连接"""
        if self.conn is not None:
            try:
                self.conn.close()
                self.conn = None
                log.info("已断开与服务器的连接")
                return True
            except Exception as e:
                log.error(f"断开连接失败: {e}")
                return False
        return True

    def send_request(
        self,
        data: bytes,
        write_timeout: float = 2.0,
        read_timeout: float = 5.0,
        total_timeout: float = 10.0,
        min_response_len: int = 1,
        retries: int = 1,
    ) -> Optional[bytes]:
        """增强版TCP请求-响应（支持超时控制和分片数据处理）

        Args:
            data: 要发送的请求数据
            write_timeout: 数据写入超时(秒)
            read_timeout: 单次recv操作的超时(秒)
            total_timeout: 整个请求-响应的总超时(秒)
            min_response_len: 最小有效响应长度
            retries: 失败重试次数

        Returns:
            bytes: 成功接收的响应数据
            None: 超时或失败时返回      
        """
        if self.conn is None:
            log.error("Not connected to server")
            return None

        original_timeout = self.conn.gettimeout()  # 保存原始超时设置
        data_buffer = bytearray()

        for attempt in range(retries + 1):
            try:
                # 0. 初始化计时器和缓冲区
                start_time = time.time()
                data_buffer.clear()

                # 1. 设置socket超时（影响后续所有操作）
                self.conn.settimeout(read_timeout)

                # 2. 带超时的数据写入
                try:
                    self.conn.sendall(data)
                    log.info(f"TX: {bytes_to_spaced_hex(data)}")
                    
                    # 捕获发送的报文
                    current_tx_id: Optional[str] = None
                    if self._message_capture:
                        current_tx_id = self._message_capture.capture_tx(data)
                except socket.timeout:
                    raise TimeoutError(f"Write timeout after {write_timeout}s")

                # 3. 接收数据（带总超时控制和分片处理）
                while (time.time() - start_time) < total_timeout:
                    try:
                        # 读取数据到缓冲区
                        chunk = self.conn.recv(1024)  # 增加缓冲区大小
                        if chunk:
                            data_buffer.extend(chunk)
                            log.info(
                                f"RX: {bytes_to_spaced_hex(chunk)} (buffer size: {len(data_buffer)})"
                            )

                            # 尝试解析缓冲区中的数据
                            try:
                                remaining_data, frame = (
                                    DLT645Protocol.deserialize_with_remaining(
                                        bytes(data_buffer)
                                    )
                                )
                                if frame is not None:
                                    log.debug(
                                        f"Successfully received complete response: {bytes_to_spaced_hex(data_buffer)}"
                                    )
                                    # 捕获接收的报文并与发送配对
                                    if self._message_capture:
                                        self._message_capture.capture_rx(bytes(data_buffer), current_tx_id)
                                    # 恢复原始超时设置
                                    self.conn.settimeout(original_timeout)
                                    return data_buffer
                            except Exception as parse_error:
                                log.warning(
                                    f"Parse error (might be incomplete data): {parse_error}"
                                )
                                # 继续等待更多数据
                                continue
                        else:  # 空数据表示连接关闭
                            log.warning("Connection closed by peer")
                            break
                    except socket.timeout:
                        # 单次recv超时，检查总超时
                        if (time.time() - start_time) >= total_timeout:
                            log.warning(f"Total timeout reached after {total_timeout}s")
                            break
                        # 继续等待更多数据
                        continue

                # 超时或中断处理
                if len(data_buffer) >= min_response_len:
                    log.warning(
                        f"Incomplete response ({len(data_buffer)} bytes): {bytes_to_spaced_hex(data_buffer)}"
                    )
                else:
                    log.error(f"No valid response within {total_timeout}s")

            except TimeoutError as e:
                log.error(str(e))
            except Exception as e:
                log.error(f"Attempt {attempt} failed: {type(e).__name__}: {str(e)}")

            # 非最后一次尝试时延迟重试
            if attempt < retries:
                log.info(f"Retrying ({attempt+1}/{retries})...")
                time.sleep(0.5 * (attempt + 1))  # 指数退避
                # 重连逻辑（如果连接已断开）
                if not self._ensure_connection():
                    continue

        # 恢复原始超时设置
        self.conn.settimeout(original_timeout)
        return None

    def _ensure_connection(self) -> bool:
        """确保连接有效（用于重试时重新连接）"""
        if self.conn is None:
            return self.connect()

        # 简单检查连接是否仍有效
        try:
            self.conn.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
            return True
        except:
            self.disconnect()
            return self.connect()

    def _safe_sendall(self, data: bytes, timeout: float) -> bool:
        """带超时的sendall实现"""
        self.conn.settimeout(timeout)
        try:
            self.conn.sendall(data)
            return True
        except socket.timeout:
            return False
        finally:
            self.conn.settimeout(self.timeout)  # 恢复原始超时

    def __enter__(self):
        """上下文管理器入口"""
        self.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """上下文管理器退出"""
        self.disconnect()




import unittest
import struct
import sys
import os
from unittest.mock import MagicMock

# Mock serial module before importing anything that might use it
sys.modules['serial'] = MagicMock()

# Ensure src is in path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from src.protocol.frame import Frame
from src.model.types.dlt645_type import CtrlCode, ErrorCode
from src.model import init_energy_def
from src.model.data.define import DIMap
# We need to make sure DIMap is populated. 
# src.model.__init__ calls init_energy_def(EnergyTypes), but we might need to trigger it.
import src.model 

class TestFrameDescription(unittest.TestCase):
    def test_read_data_request(self):
        # 0x00010000: (当前)正向有功总电能
        # DI is little endian: 00 00 01 00
        di_val = 0x00010000
        di_bytes = struct.pack("<I", di_val)
        
        frame = Frame(ctrl_code=CtrlCode.ReadData, data=bytearray(di_bytes))
        
        desc = frame.description
        print(f"Read Request Desc: {desc}")
        self.assertIn("读取", desc)
        self.assertIn("正向有功总电能", desc)
        self.assertIn("(DI=00010000)", desc) # Check format

    def test_read_data_response(self):
        di_val = 0x00010000
        di_bytes = struct.pack("<I", di_val)
        # Mocking some data return
        data = bytearray(di_bytes) + bytearray([0x12, 0x34, 0x56, 0x78]) 
        
        frame = Frame(ctrl_code=CtrlCode.ReadData | 0x80, data=data)
        desc = frame.description
        print(f"Read Response Desc: {desc}")
        self.assertIn("读取", desc)
        self.assertIn("响应", desc)
        self.assertIn("正向有功总电能", desc)
        self.assertIn("(DI=00010000)", desc)

    def test_write_data_request(self):
        di_val = 0x04000401 # Check a parameter DI
        di_bytes = struct.pack("<I", di_val)
        data = bytearray(di_bytes) + bytearray([0x11, 0x11, 0x11, 0x11]) # Password + ...
        
        frame = Frame(ctrl_code=CtrlCode.WriteData, data=data)
        desc = frame.description
        print(f"Write Request Desc: {desc}")
        self.assertIn("写入", desc)
        self.assertIn("(DI=04000401)", desc)

    def test_error_response(self):
        # CtrlCode.ReadData | 0xC0 = Error response for ReadData
        # Error bit 0x01 = Other Error? No, 0x01 is often specific.
        # ErrorCode.RequestDataEmpty = 0x01 ? Let's ignore exact match and check string.
        
        frame = Frame(ctrl_code=CtrlCode.ReadData | 0xC0, data=bytearray([0x01]))
        desc = frame.description
        print(f"Error Response Desc: {desc}")
        self.assertIn("读取数据失败响应", desc)

    def test_unknown_di(self):
        di_val = 0xFFFFFFFF
        di_bytes = struct.pack("<I", di_val)
        frame = Frame(ctrl_code=CtrlCode.ReadData, data=bytearray(di_bytes))
        desc = frame.description
        print(f"Unknown DI Desc: {desc}")
        self.assertIn("未知数据项(DI=FFFFFFFF)", desc)

if __name__ == '__main__':
    unittest.main()



import sys
import time

sys.path.append("../..")
from src.service.clientsvc.client_service import MeterClientService

if __name__ == "__main__":
    client_svc = MeterClientService.new_rtu_client("/dev/ttyV1", 9600, 8, 1, "N", 5.0)
    if not client_svc:
        print("创建客户端失败")
        sys.exit(1)

    # 设置设备密码(0级)
    client_svc.set_password("00123456")

    # 读取通讯地址
    print("读取通讯地址...")
    address_data = client_svc.read_address()
    if address_data and hasattr(address_data, "value"):
        print(f"通讯地址: {address_data.value}")
    else:
        print("读取通讯地址失败")
        sys.exit(1)

    # 设置设备地址
    client_svc.set_address(address_data.value)

    # 读取电能数据
    data_item = client_svc.read_00(0x00000000)
    print(f"电能数据: {data_item}")

    # 读取最大需量及发生时间
    data_item2 = client_svc.read_01(0x01010000)
    print(f"最大需量及发生时间: {data_item2}")

    # 读取变量数据
    data_item3 = client_svc.read_02(0x02010100)
    print(f"变量数据: {data_item3}")

    # 读取事件记录数据
    data_item4 = client_svc.read_03(0x03010000)
    print(f"事件记录数据: {data_item4}")

    # 读取参变量
    data_item5 = client_svc.read_04(0x04000101)
    print(f"日期及星期: {data_item5}")

    data_item6 = client_svc.read_04(0x04000204)
    print(f"费率数: {data_item6}")

    # 读取时区表数据
    data_item7 = client_svc.read_04(0x04010000)
    for item in data_item7:
        print(item)

    # 读取公共假日日期及时段表号
    data_item8 = client_svc.read_04(0x04030001)
    print(f"公共假日日期及时段表号: {data_item8}")

    # 修改密码
    client_svc.change_password("00123456", "04123456")

    # 写参变量
    data_item9 = client_svc.write_04(
        0x04000101, "25120901", password="04123456"
    )  # 写日期及星期



# 数据标识DI（完整定义）
from typing import List
from ....model.types.data_type import DataItem, DataFormat
from . import DIMap

energy_di_list = [
    0x00800000,
    0x00810000,
    0x00820000,
    0x00830000,
    0x00840000,
    0x00850000,
    0x00860000,
    0x00150000,
    0x00160000,
    0x00170000,
    0x00180000,
    0x00190000,
    0x001A0000,
    0x001B0000,
    0x001C0000,
    0x001D0000,
    0x001E0000,
    0x00940000,
    0x00950000,
    0x00960000,
    0x00970000,
    0x00980000,
    0x00990000,
    0x009A0000,
    0x00290000,
    0x002A0000,
    0x002B0000,
    0x002C0000,
    0x002D0000,
    0x002E0000,
    0x002F0000,
    0x00300000,
    0x00310000,
    0x00320000,
    0x00A80000,
    0x00A90000,
    0x00AA0000,
    0x00AB0000,
    0x00AC0000,
    0x00AD0000,
    0x00AE0000,
    0x003D0000,
    0x003E0000,
    0x003F0000,
    0x00400000,
    0x00410000,
    0x00420000,
    0x00430000,
    0x00440000,
    0x00450000,
    0x00460000,
    0x00BC0000,
    0x00BD0000,
    0x00BE0000,
    0x00BF0000,
    0x00C00000,
    0x00C10000,
    0x00C20000,
]


def init_energy_def(energy_types: List[DataItem]):
    """初始化电能定义
    
    Args:
        energy_types: 电能类型列表，包含正向有功电能、反向有功电能等类型
    """
    di3 = 0  # 数据类型
    di2 = 0  # 电能类型
    di1 = 0  # 同一类型电能里的不同项
    di0 = 0  # 结算日

    for i in range(64):
        for j in range(13):
            if j == 0:
                name_prefix = "（当前）"
            else:
                name_prefix = f"（上{j}结算日）"

            # 组合有功费率电能
            DIMap[(di3 << 24) | (di2 << 16) | ((di1 + i) << 8) | (di0 + j)] = DataItem(
                di=(di3 << 24) | (di2 << 16) | ((di1 + i) << 8) | (di0 + j),
                name=name_prefix + energy_types[i].name,
                data_format=DataFormat.XXXXXX_XX.value,
                unit=energy_types[i].unit,
            )

            # 正向有功费率电能
            DIMap[(di3 << 24) | ((di2 + 1) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 1) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 + i].unit,
                )
            )

            # 反向有功费率电能
            DIMap[(di3 << 24) | ((di2 + 2) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 2) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 2 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 2 + i].unit,
                )
            )

            # 组合无功1费率电能
            DIMap[(di3 << 24) | ((di2 + 3) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 3) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 3 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 3 + i].unit,
                )
            )

            # 组合无功2费率电能地址
            DIMap[(di3 << 24) | ((di2 + 4) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 4) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 4 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 4 + i].unit,
                )
            )

            # 第一象限无功电能
            DIMap[(di3 << 24) | ((di2 + 5) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 5) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 5 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 5 + i].unit,
                )
            )

            # 第二象限无功电能
            DIMap[(di3 << 24) | ((di2 + 6) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 6) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 6 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 6 + i].unit,
                )
            )

            # 第三象限无功电能
            DIMap[(di3 << 24) | ((di2 + 7) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 7) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 7 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 7 + i].unit,
                )
            )

            # 第四象限无功电能
            DIMap[(di3 << 24) | ((di2 + 8) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 8) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 8 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 8 + i].unit,
                )
            )

            # 正向视在电能
            DIMap[(di3 << 24) | ((di2 + 9) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 9) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 9 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 9 + i].unit,
                )
            )

            # 反向视在电能
            DIMap[(di3 << 24) | ((di2 + 10) << 16) | ((di1 + i) << 8) | (di0 + j)] = (
                DataItem(
                    di=(di3 << 24) | ((di2 + 10) << 16) | ((di1 + i) << 8) | (di0 + j),
                    name=name_prefix + energy_types[64 * 10 + i].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 10 + i].unit,
                )
            )

            # 最后几个数据特殊处理
            for k in range(len(energy_di_list)):
                value = (energy_di_list[k] & 0xFFFFFF00) | (
                    di0 + j
                )  # 提取energyDiList中的前24位，然后添加结算日信息（最后8位）
                DIMap[value] = DataItem(
                    di=value,
                    name=name_prefix + energy_types[64 * 11 + k].name,
                    data_format=DataFormat.XXXXXX_XX.value,
                    unit=energy_types[64 * 11 + k].unit,
                )



import os

# 在包安装模式下，使用__file__获取目前模块的路径
current_path = os.path.dirname(os.path.abspath(__file__))
# dlt645/common/env.py -> dlt645/
package_root = os.path.dirname(current_path)
conf_path = os.path.join(package_root, 'config')
log_path = os.path.join(package_root, 'log')

# 如果配置目录不存在，尝试使用相对路径（开发模式）
if not os.path.exists(conf_path):
    # 开发模式下的路径结构
    root_path = os.path.dirname(os.path.dirname(current_path))
    log_path = os.path.join(root_path, 'log')
    src_path = os.path.join(root_path, 'src')
    conf_path = os.path.join(src_path, 'config')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
DLT645协议库基本测试

这个脚本提供了基本的单元测试，确保包能正常工作
"""

import unittest
import sys
import os

# 添加python目录到Python路径（test的上一级目录）
# 使用 abspath 确保在任何执行环境下都能正确解析路径
python_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
sys.path.insert(0, python_dir)

try:
    from src.service.serversvc.server_service import MeterServerService
    from src.service.clientsvc.client_service import MeterClientService
    from src.protocol.protocol import DLT645Protocol
    from src.model.types.dlt645_type import CtrlCode
except ImportError as e:
    print(f"导入模块失败: {e}")
    print("请确保在dlt645包目录中运行此脚本")
    sys.exit(1)


class TestDLT645Basic(unittest.TestCase):
    """DLT645协议基本功能测试"""

    def test_import(self):
        """测试模块导入"""
        print("测试模块导入...")
        self.assertIsNotNone(MeterServerService)
        self.assertIsNotNone(MeterClientService)
        self.assertIsNotNone(DLT645Protocol)
        self.assertIsNotNone(CtrlCode)
        print("✓ 模块导入正常")

    def test_tcp_server_creation(self):
        """测试TCP服务器创建"""
        print("测试TCP服务器创建...")
        try:
            server = MeterServerService.new_tcp_server("127.0.0.1", 8022, 30)
            self.assertIsNotNone(server)
            self.assertIsInstance(server, MeterServerService)
            print("✓ TCP服务器创建成功")
        except Exception as e:
            self.fail(f"TCP服务器创建失败: {e}")

    def test_tcp_client_creation(self):
        """测试TCP客户端创建"""
        print("测试TCP客户端创建...")
        try:
            client = MeterClientService.new_tcp_client("127.0.0.1", 8023, 10.0)
            self.assertIsNotNone(client)
            self.assertIsInstance(client, MeterClientService)
            print("✓ TCP客户端创建成功")
        except Exception as e:
            self.fail(f"TCP客户端创建失败: {e}")

    def test_server_data_setting(self):
        """测试服务器数据设置"""
        print("测试服务器数据设置...")
        try:
            server = MeterServerService.new_tcp_server("127.0.0.1", 8024, 30)

            # 测试设置电能量数据
            result = server.set_00(0x00000000, 123.45)
            # 注意：由于没有实际的数据存储后端，这里可能返回False
            # 我们主要测试方法调用不会抛出异常

            # 测试设置变量数据
            result = server.set_02(0x02010100, 220.0)

            # 测试设置地址
            server.set_address("010203040506")

            print("✓ 服务器数据设置方法调用正常")
        except Exception as e:
            self.fail(f"服务器数据设置失败: {e}")

    def test_client_address_setting(self):
        """测试客户端地址设置"""
        print("测试客户端地址设置...")
        try:
            client = MeterClientService.new_tcp_client("127.0.0.1", 8025, 10.0)

            # 测试设置地址
            result = client.set_address("010203040506")
            self.assertTrue(result)

            # 测试设置密码
            result = client.set_password("00000000")
            self.assertTrue(result)

            print("✓ 客户端地址和密码设置正常")
        except Exception as e:
            self.fail(f"客户端地址设置失败: {e}")

    def test_protocol_constants(self):
        """测试协议常量"""
        print("测试协议常量...")
        try:
            # 测试控制码常量
            self.assertIsNotNone(CtrlCode.ReadData)
            self.assertIsNotNone(CtrlCode.ReadAddress)
            self.assertIsNotNone(CtrlCode.WriteAddress)

            print("✓ 协议常量定义正常")
        except Exception as e:
            self.fail(f"协议常量测试失败: {e}")


def run_tests():
    """运行测试"""
    print("DLT645协议库基本测试")
    print("=" * 50)

    # 创建测试套件
    suite = unittest.TestLoader().loadTestsFromTestCase(TestDLT645Basic)

    # 运行测试
    runner = unittest.TextTestRunner(verbosity=2)
    result = runner.run(suite)

    print("\n" + "=" * 50)
    if result.wasSuccessful():
        print("✓ 所有测试通过！")
        print("包基本功能正常，可以进行安装和使用。")
    else:
        print("✗ 部分测试失败！")
        print("请检查错误信息并修复问题。")

    return result.wasSuccessful()


if __name__ == "__main__":
    success = run_tests()
    sys.exit(0 if success else 1)



"""DLT645 服务端服务模块。

本模块实现了 DLT645 协议的服务端业务服务功能，包括：
- 处理客户端数据读取请求
- 处理客户端数据写入请求
- 通讯地址管理
- 密码验证和管理
"""

import struct
from typing import Optional, Union, List

from ...common.transform import (
    bytes_to_spaced_hex,
    float_to_bcd,
    datetime_to_bcd,
    string_to_bcd,
    bcd_to_value,
    bcd_to_string,
)
from ...model.data.data_handler import set_data_item, get_data_item
from ...model.types.data_type import DataItem
from ...model.validators import validate_device
from ...model.types.dlt645_type import (
    DI_LEN,
    PASSWORD_LEN,
    ADDRESS_LEN,
    OPERATOR_CODE_LEN,
    CtrlCode,
    Demand,
    ErrorCode,
    EventRecord,
    PasswordManager,
)
from ...protocol.protocol import DLT645Protocol
from ...model.data import data_handler as data
from ...service.serversvc.log import log
from ...transport.server.rtu_server import RtuServer
from ...transport.server.tcp_server import TcpServer
from ...common.message_capture import MessageCapture
from ...common.message_types import MessageRecord, MessagePair
from typing import List


class MeterServerService:
    """电表服务端服务类。

    用于模拟 DLT645 电表设备，响应客户端的数据读写请求。

    :ivar server: 通信服务器（TCP 或 RTU）。
    :ivar address: 设备地址（6字节）。
    :ivar password_manager: 密码管理器。
    :ivar clear_meter_event_records: 电表清零事件记录列表。
    :ivar event_records: 事件记录列表。
    """

    def __init__(
        self,
        server: Union[TcpServer, RtuServer],
        address: Optional[bytearray] = bytearray([0x00, 0x00, 0x00, 0x00, 0x00, 0x00]),
        password_manager: Optional[PasswordManager] = PasswordManager(),
    ):
        """初始化电表服务端服务。

        :param server: 通信服务器实例（TcpServer 或 RtuServer）。
        :type server: Union[TcpServer, RtuServer]
        :param address: 设备地址，默认为全零。
        :type address: Optional[bytearray]
        :param password_manager: 密码管理器，默认创建新实例。
        :type password_manager: Optional[PasswordManager]
        """
        self.server = server
        self.address = address
        self.password_manager = password_manager
        self.clear_meter_event_records = []  # 记录电表清零事件
        self.event_records = []  # 记录事件

    @classmethod
    def new_tcp_server(
        cls, ip: str, port: int, timeout: float = 5.0
    ) -> "MeterServerService":
        """创建 TCP 服务器

        :param ip: IP 地址
        :param port: 端口
        :param timeout: 超时时间
        :return:
        """
        # 1. 先创建 TcpServer
        tcp_server = TcpServer(ip, port, timeout, None)
        # 2. 创建 MeterServerService，注入 TcpServer（作为 Server 接口）
        return cls.new_meter_server_service(tcp_server)

    @classmethod
    def new_rtu_server(
        cls,
        port: str,
        data_bits: int,
        stop_bits: int,
        baud_rate: int,
        parity: str,
        timeout: float,
    ) -> "MeterServerService":
        """创建 RTU 服务器

        :param port: 端口
        :param data_bits: 数据位
        :param stop_bits: 停止位
        :param baud_rate: 波特率
        :param parity: 校验位
        :param timeout: 超时时间
        :return:
        """
        # 1. 先创建 RtuServer
        rtu_server = RtuServer(port, data_bits, stop_bits, baud_rate, parity, timeout)
        # 2. 创建 MeterServerService，注入 RtuServer（作为 Server 接口）
        return cls.new_meter_server_service(rtu_server)

    @classmethod
    def new_meter_server_service(
        cls, server: Union[TcpServer, RtuServer]
    ) -> "MeterServerService":
        """创建新的MeterServerService实例

        :param server: 服务器实例（TCP或RTU）
        :return: MeterServerService实例
        """
        # 创建业务服务实例
        meter_service = cls(server)
        # 将服务实例注入回服务器
        server.service = meter_service
        return meter_service

    # 设置时间，需根据实际情况实现
    def set_time(self, data_bytes):
        pass

    def set_address(self, address: str):
        """写通讯地址

        :param address:
        :return:
        """
        address = string_to_bcd(address)
        if len(address) != ADDRESS_LEN:
            raise ValueError("invalid address length")
        self.address = address

    def set_password(self, password: str) -> None:
        """写密码

        :param password:
        :return:
        """
        password = string_to_bcd(password)
        self.password_manager.set_password(password)
        log.info(f"设置密码: {bytes_to_spaced_hex(password)}")

    def set_00(self, di: int, value: float) -> bool:
        """写电能量

        :param di: 数据项
        :param value: 值
        :return:
        """
        ok = set_data_item(di, value)
        if not ok:
            log.error(f"写电能量失败")
        return ok

    def set_01(self, di: int, demand: Demand) -> bool:
        """写最大需量及发生时间

        :param di: 数据项
        :param demand: 值
        :return:
        """
        ok = set_data_item(di, demand)
        if not ok:
            log.error(f"写最大需量及发生时间失败")
        return ok

    def set_02(self, di: int, value: float) -> bool:
        """写变量

        :param di: 数据项
        :param value: 值
        :return:
        """
        data_item = get_data_item(di)
        if data_item is None:
            log.error(f"获取变量数据项失败")
            return False

        ok = set_data_item(di, value)
        if not ok:
            log.error(f"写变量失败")
            return False
        return ok

    def set_03(self, di: int, value: list[str, tuple[str, str]]) -> bool:
        """写事件记录

        :param di: 数据项
        :param value: 值
        :return:
        """
        data_item = get_data_item(di)
        if data_item is None:
            log.error(f"获取事件记录数据项失败")
            return False

        if not set_data_item(di, value):
            log.error(f"写事件记录失败")
            return False
        return True

    def set_04(self, di: int, value: str | list) -> bool:
        """写参变量

        :param di: 数据项
        :param value: 值
        :return:
        """
        data_item = get_data_item(di)
        if data_item is None:
            log.error(f"获取参变量数据项失败")
            return False

        if not set_data_item(di, value):
            log.error(f"写参变量失败")
            return False
        return True

    def get_data_item(self, di: int) -> Optional[DataItem]:
        """获取数据项

        :param di: 数据项
        :return:
        """
        return get_data_item(di)

    def handle_request(self, frame):
        """处理读数据请求

        :param frame:
        :return:
        """
        try:
            # 1. 验证设备
            if not validate_device(self.address, frame.ctrl_code, frame.addr):
                log.error(f"验证设备地址: {bytes_to_spaced_hex(frame.addr)} 失败")
                # 返回未授权异常帧
                return self._build_error_response(
                    frame, error_code=ErrorCode.OtherError
                )

            # 2. 根据控制码判断请求类型
            if frame.ctrl_code == CtrlCode.BroadcastTimeSync:  # 广播校时
                log.info(f"广播校时: {bytes_to_spaced_hex(frame.data)}")
                self.set_time(frame.data)
                return DLT645Protocol.build_frame(
                    frame.addr, frame.ctrl_code | 0x80, frame.data
                )
            elif frame.ctrl_code == CtrlCode.ReadData:
                # 解析数据标识
                di = frame.data
                di3 = di[3]
                if di3 == 0x00:  # 读取电能
                    # 构建响应帧
                    res_data = bytearray(8)
                    # 解析数据标识为 32 位无符号整数
                    data_id = struct.unpack("<I", frame.data[:DI_LEN])[0]
                    data_item = get_data_item(data_id)
                    if data_item is None:
                        log.error(f"数据项未找到: {data_id}")
                        return self._build_error_response(
                            frame, error_code=ErrorCode.RequestDataEmpty
                        )
                    res_data[:DI_LEN] = frame.data[:DI_LEN]  # 仅复制前 4 字节数据标识
                    value = data_item.value
                    # 转换为 BCD 码
                    bcd_value = float_to_bcd(value, data_item.data_format, "little")
                    res_data[DI_LEN:] = bcd_value
                    return DLT645Protocol.build_frame(
                        frame.addr, frame.ctrl_code | 0x80, bytes(res_data)
                    )
                elif di3 == 0x01:  # 读取最大需量及发生时间
                    res_data = bytearray(12)
                    # 解析数据标识为 32 位无符号整数
                    data_id = struct.unpack("<I", frame.data[:DI_LEN])[0]
                    data_item = get_data_item(data_id)
                    if data_item is None:
                        log.error(f"数据项未找到: {data_id}")
                        return self._build_error_response(
                            frame, error_code=ErrorCode.RequestDataEmpty
                        )
                    res_data[:DI_LEN] = frame.data[:DI_LEN]  # 返回数据标识
                    demand: Demand = data_item.value
                    # 转换为 BCD 码
                    bcd_value = float_to_bcd(
                        demand.value, data_item.data_format, "little"
                    )
                    res_data[DI_LEN : DI_LEN + 3] = bcd_value[:3]
                    # 需量发生时间
                    res_data[DI_LEN + 3 : 12] = datetime_to_bcd(demand.time)
                    return DLT645Protocol.build_frame(
                        frame.addr, frame.ctrl_code | 0x80, bytes(res_data)
                    )
                elif di3 == 0x02:  # 读变量
                    # 解析数据标识为 32 位无符号整数
                    data_id = struct.unpack("<I", frame.data[:DI_LEN])[0]
                    data_item = get_data_item(data_id)
                    if data_item is None:
                        log.error(f"数据项未找到: {data_id}")
                        return self._build_error_response(
                            frame, error_code=ErrorCode.RequestDataEmpty
                        )
                    # 变量数据长度
                    data_len = DI_LEN
                    data_len += (
                        len(data_item.data_format) - 1
                    ) // 2  # (数据格式长度 - 1 位小数点)/2
                    # 构建响应帧
                    res_data = bytearray(data_len)
                    res_data[:DI_LEN] = frame.data[:DI_LEN]  # 仅复制前 DI_LEN 字节
                    value = data_item.value
                    # 转换为 BCD 码（小端序）
                    bcd_value = float_to_bcd(value, data_item.data_format, "little")
                    res_data[DI_LEN:data_len] = bcd_value
                    return DLT645Protocol.build_frame(
                        frame.addr, frame.ctrl_code | 0x80, bytes(res_data)
                    )
                elif di3 == 0x03:  # 读事件记录
                    # 解析数据标识为 32 位无符号整数
                    data_id = struct.unpack("<I", frame.data[:DI_LEN])[0]
                    data_item: Optional[List[DataItem]] = get_data_item(data_id)
                    if data_item is None:
                        log.error(f"数据项未找到: {data_id}")
                        return self._build_error_response(
                            frame, error_code=ErrorCode.RequestDataEmpty
                        )

                    res_data = bytearray()
                    res_data.extend(frame.data[:DI_LEN])  # 仅复制前 DI_LEN 字节
                    for item in data_item:
                        event_record: EventRecord = item.value
                        if isinstance(event_record.event, tuple):
                            for event_item in reversed(event_record.event):
                                value = string_to_bcd(event_item, "little")
                                res_data.extend(value)
                        elif isinstance(event_record.event, str):
                            value = string_to_bcd(event_record.event, "little")
                            res_data.extend(value)
                        elif isinstance(event_record.event, float):
                            value = float_to_bcd(event_record.event, "little")
                            res_data.extend(value)
                    return DLT645Protocol.build_frame(
                        frame.addr, frame.ctrl_code | 0x80, bytes(res_data)
                    )
                elif di3 == 0x04:  # 读参变量
                    # 解析数据标识为 32 位无符号整数
                    data_id = struct.unpack("<I", frame.data[:DI_LEN])[0]
                    data_item = get_data_item(data_id)
                    if data_item is None:
                        log.error(f"数据项未找到: {data_id}")
                        return self._build_error_response(
                            frame, error_code=ErrorCode.RequestDataEmpty
                        )

                    # 变量数据长度
                    data_len = DI_LEN
                    # 时段表数据
                    if (
                        0x04010000
                        <= int.from_bytes(di, byteorder="little")
                        <= 0x04020008
                    ):
                        res_data = bytearray(DI_LEN + len(data_item) * 2)
                        for i, item in enumerate(data_item):
                            step = len(item.data_format) // 2
                            data_len += step
                            res_data[:DI_LEN] = frame.data[:DI_LEN]  # 复制数据标识
                            value = item.value
                            bcd_value = string_to_bcd(value, "little")

                            # 扩展res_data以容纳BCD数据
                            res_data[DI_LEN + step * i : DI_LEN + step * (i + 1)] = (
                                bcd_value
                            )
                        return DLT645Protocol.build_frame(
                            frame.addr, frame.ctrl_code | 0x80, bytes(res_data)
                        )
                    else:
                        # 根据数据格式确定数据长度
                        data_format = data_item.data_format
                        data_len += len(data_format) // 2

                        # 构建响应帧
                        res_data = bytearray(data_len)
                        res_data[:DI_LEN] = frame.data[:DI_LEN]  # 复制数据标识
                        value = data_item.value

                        bcd_value = string_to_bcd(value, "little")

                        # 扩展res_data以容纳BCD数据
                        res_data[DI_LEN : DI_LEN + data_len] = bcd_value

                        return DLT645Protocol.build_frame(
                            frame.addr, frame.ctrl_code | 0x80, bytes(res_data)
                        )
                else:
                    log.error(f"未知的数据标识类型: {hex(di3)}")
                    return self._build_error_response(
                        frame, error_code=ErrorCode.OtherError
                    )
            elif frame.ctrl_code == CtrlCode.WriteData:
                log.debug(f"收到写数据请求: {bytes_to_spaced_hex(frame.data)}")
                # 解析数据标识为 32 位无符号整数
                data_id = struct.unpack("<I", frame.data[:DI_LEN])[0]
                data_item = get_data_item(data_id)
                if data_item is None:
                    log.error(f"数据项未找到: {data_id}")
                    return self._build_error_response(
                        frame, error_code=ErrorCode.RequestDataEmpty
                    )

                # 提取密码
                password = frame.data[DI_LEN : DI_LEN + PASSWORD_LEN]
                if not self.password_manager.check_password(password):
                    log.error(f"密码错误: {bytes_to_spaced_hex(password)}")
                    return self._build_error_response(
                        frame, error_code=ErrorCode.AuthFailed
                    )

                # 提取操作者代码
                operator_code = frame.data[
                    DI_LEN + PASSWORD_LEN : DI_LEN + PASSWORD_LEN + OPERATOR_CODE_LEN
                ]

                # 提取数据
                data_len = len(frame.data) - DI_LEN - PASSWORD_LEN
                data = frame.data[
                    DI_LEN
                    + PASSWORD_LEN
                    + OPERATOR_CODE_LEN : DI_LEN
                    + PASSWORD_LEN
                    + OPERATOR_CODE_LEN
                    + data_len
                ]
                # 解析数据
                value = bcd_to_value(data, data_item.data_format, "little")
                if not set_data_item(data_id, value):
                    log.error(f"设置数据项 {data_id} 失败")
                    return self._build_error_response(
                        frame, error_code=ErrorCode.OtherError
                    )

                # 构建响应帧
                res_data = bytearray()  # 广播校时不需要返回数据
                return DLT645Protocol.build_frame(
                    frame.addr, frame.ctrl_code | 0x80, res_data
                )
            elif frame.ctrl_code == CtrlCode.ReadAddress:
                # 构建响应帧
                res_data = self.address[:ADDRESS_LEN]
                return DLT645Protocol.build_frame(
                    bytes(self.address), frame.ctrl_code | 0x80, bytes(res_data)
                )
            elif frame.ctrl_code == CtrlCode.WriteAddress:
                res_data = bytearray()
                # 解析数据
                addr = frame.data[:ADDRESS_LEN]
                self.set_address(addr)  # 设置通讯地址
                return DLT645Protocol.build_frame(
                    bytes(self.address), frame.ctrl_code | 0x80, res_data
                )
            elif frame.ctrl_code == CtrlCode.ChangePassword:
                # 解析数据
                res_data = bytearray()
                old_password = frame.data[DI_LEN : DI_LEN + PASSWORD_LEN]
                new_password = frame.data[
                    DI_LEN + PASSWORD_LEN : DI_LEN + PASSWORD_LEN * 2
                ]
                if not self.password_manager.change_password(
                    old_password, new_password
                ):
                    log.error(f"修改密码失败")
                    return self._build_error_response(
                        frame, error_code=ErrorCode.AuthFailed
                    )
                res_data = new_password  # 返回新密码响应
                return DLT645Protocol.build_frame(
                    bytes(self.address), frame.ctrl_code | 0x80, res_data
                )
            elif frame.ctrl_code == CtrlCode.ClearDemand:
                log.debug(f"收到需量清零请求: {bytes_to_spaced_hex(frame.data)}")
                # 解析数据标识为 32 位无符号整数
                data_id = struct.unpack("<I", frame.data[:DI_LEN])[0]
                data_item = get_data_item(data_id)
                if data_item is None:
                    log.error(f"数据项未找到: {data_id}")
                    return self._build_error_response(
                        frame, error_code=ErrorCode.RequestDataEmpty
                    )

                # 提取密码
                password = frame.data[DI_LEN : DI_LEN + PASSWORD_LEN]
                if not self.password_manager.check_password(password):
                    log.error(f"密码错误: {bytes_to_spaced_hex(password)}")
                    return self._build_error_response(
                        frame, error_code=ErrorCode.AuthFailed
                    )

                # 提取操作者代码
                operator_code = frame.data[
                    DI_LEN + PASSWORD_LEN : DI_LEN + PASSWORD_LEN + OPERATOR_CODE_LEN
                ]

                # 执行需量清零：将需量值设为0，时间设为当前时间
                from datetime import datetime
                cleared_demand = Demand(0.0, datetime.now())
                if not set_data_item(data_id, cleared_demand):
                    log.error(f"需量清零失败: {data_id}")
                    return self._build_error_response(
                        frame, error_code=ErrorCode.OtherError
                    )

                log.info(f"需量清零成功: DI={hex(data_id)}, 操作者代码={bytes_to_spaced_hex(operator_code)}")

                # 构建响应帧
                res_data = bytearray()
                return DLT645Protocol.build_frame(
                    frame.addr, frame.ctrl_code | 0x80, res_data
                )
            else:
                log.error(f"未知的控制码: {hex(frame.ctrl_code)}")
                return self._build_error_response(
                    frame, error_code=ErrorCode.OtherError
                )
        except Exception as e:
            # 捕获其他未预期的异常
            log.error(f"处理请求时发生未预期异常: {str(e)}")
            # 返回通用错误异常帧
            return self._build_error_response(frame, error_code=ErrorCode.OtherError)

    def _build_error_response(self, frame, error_code: int):
        """构建异常响应帧

        :param frame: 原始请求帧
        :param error_code: 错误码
        :return: 异常响应帧
        """
        # 构建异常响应帧，控制码最高位设为1表示响应
        log.debug(
            f"构建异常响应帧: 地址={frame.addr.hex()}, 控制码={hex(frame.ctrl_code | 0xC0)}, 错误码={error_code}"
        )
        error_data = bytes([error_code])
        return DLT645Protocol.build_frame(  # D7=1, D6=1表示异常响应, C=1100
            frame.addr, frame.ctrl_code | 0xC0, error_data
        )

    # ==================== 报文捕获方法 ====================

    def enable_message_capture(self, queue_size: int = 100) -> None:
        """启用报文捕获功能。

        :param queue_size: 报文队列大小，默认100
        :type queue_size: int
        """
        if self.server._message_capture is None:
            self.server._message_capture = MessageCapture(enabled=True, queue_size=queue_size)
        else:
            self.server._message_capture.enable()
            self.server._message_capture.set_queue_size(queue_size)
        log.info(f"报文捕获已启用，队列大小: {queue_size}")

    def disable_message_capture(self) -> None:
        """禁用报文捕获功能。"""
        if self.server._message_capture:
            self.server._message_capture.disable()
        log.info("报文捕获已禁用")

    def get_captured_messages(self, count: int = 0) -> List[MessageRecord]:
        """获取捕获的报文列表。

        :param count: 要获取的数量，0表示全部
        :type count: int
        :return: 报文列表
        :rtype: List[MessageRecord]
        """
        if self.server._message_capture:
            return self.server._message_capture.get_all_messages(count)
        return []

    def get_captured_tx_messages(self, count: int = 0) -> List[MessageRecord]:
        """获取捕获的发送报文列表。

        :param count: 要获取的数量，0表示全部
        :type count: int
        :return: 发送报文列表
        :rtype: List[MessageRecord]
        """
        if self.server._message_capture:
            return self.server._message_capture.get_tx_messages(count)
        return []

    def get_captured_rx_messages(self, count: int = 0) -> List[MessageRecord]:
        """获取捕获的接收报文列表。

        :param count: 要获取的数量，0表示全部
        :type count: int
        :return: 接收报文列表
        :rtype: List[MessageRecord]
        """
        if self.server._message_capture:
            return self.server._message_capture.get_rx_messages(count)
        return []

    def get_captured_pairs(self, count: int = 0) -> List[MessagePair]:
        """获取捕获的TX/RX配对列表。

        :param count: 要获取的数量，0表示全部
        :type count: int
        :return: 配对列表
        :rtype: List[MessagePair]
        """
        if self.server._message_capture:
            return self.server._message_capture.get_pairs(count)
        return []

    def clear_captured_messages(self) -> None:
        """清空所有捕获的报文。"""
        if self.server._message_capture:
            self.server._message_capture.clear()
        log.info("捕获的报文已清空")

    def get_message_capture_stats(self) -> dict:
        """获取报文捕获统计信息。

        :return: 统计信息字典
        :rtype: dict
        """
        if self.server._message_capture:
            return self.server._message_capture.get_stats()
        return {"enabled": False}



# 初始化数据类型
import os

from ..common.env import conf_path
from ..model.data.define.demand_def import init_demand_def
from ..model.data.define.energy_def import init_energy_def
from ..model.data.define.variable_def import init_variable_def
from ..model.data.define.parameter_def import init_parameter_def
from ..model.data.define.event_record_def import init_event_record_def
from ..model.types.data_type import init_data_type_from_json

EnergyTypes = []
DemandTypes = []
VariableTypes = []
ParaMeterTypes = []
EventRecordTypes = []


def init():
    global EnergyTypes, DemandTypes, VariableTypes, ParaMeterTypes, EventRecordTypes
    EnergyTypes = init_data_type_from_json(os.path.join(conf_path, "energy_types.json"))
    DemandTypes = init_data_type_from_json(os.path.join(conf_path, "demand_types.json"))
    VariableTypes = init_data_type_from_json(
        os.path.join(conf_path, "variable_types.json")
    )
    ParaMeterTypes = init_data_type_from_json(
        os.path.join(conf_path, "parameter_types.json")
    )
    EventRecordTypes = init_data_type_from_json(
        os.path.join(conf_path, "event_record_types.json")
    )

    init_energy_def(EnergyTypes)
    init_demand_def(DemandTypes)
    init_variable_def(VariableTypes)
    init_parameter_def(ParaMeterTypes)
    init_event_record_def(EventRecordTypes)


# 执行初始化
init()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information
import os
import sys
# 添加项目根目录到 Python 路径
sys.path.insert(0, os.path.abspath('../..'))

project = 'dlt645'
copyright = '2026, 陈东宇'
author = '陈东宇'
release = 'v1.4.0'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

templates_path = ['_templates']
exclude_patterns = []

language = 'zh'

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'alabaster'
html_static_path = ['_static']

# 扩展模块
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx.ext.coverage',
    'sphinx.ext.mathjax',
    'sphinx.ext.viewcode',
    'sphinx.ext.githubpages',
    'sphinx_autodoc_typehints',
    'sphinx_copybutton',
]

# 模板路径
templates_path = ['_templates']

# 排除的文件
exclude_patterns = []

# 主文档名称
master_doc = 'index'

# -- Options for HTML output -------------------------------------------------

# 主题名称
html_theme = 'sphinx_rtd_theme'

# 主题配置选项
html_theme_options = {
    # 导航菜单默认展开，不折叠
    'collapse_navigation': False,
    # 显示完整的目录树结构
    'navigation_depth': -1,
}

# 静态资源路径
html_static_path = ['_static']

# -- Options for autodoc ----------------------------------------------------

# 自动文档生成选项
autodoc_member_order = 'bysource'
autoclass_content = 'both'

# -- Options for intersphinx -------------------------------------------------

# 交叉引用配置
intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
}


"""
数据转换工具模块。

本模块提供了 DLT645 协议所需的各种数据转换功能，包括：
- 字节数组与整数/浮点数之间的转换
- BCD 码与数值/字符串之间的转换
- 日期时间与 BCD 码之间的转换
- 十六进制格式化等辅助功能
"""

import binascii
import struct
from datetime import datetime
from typing import Union


def pad_with_zeros(length: int) -> str:
    """根据指定长度生成补零字符串。

    :param length: 需要生成的零的个数。
    :type length: int
    :return: 由指定数量零组成的字符串。
    :rtype: str
    """
    return "0" * length


def bytes_to_int(byte_array: bytearray) -> int:
    """将字节数组转换为无符号整数。

    使用小端序进行转换，如果字节数组长度不足4字节，
    会在右侧补零至4字节。

    :param byte_array: 待转换的字节数组。
    :type byte_array: bytearray
    :return: 转换后的无符号整数。
    :rtype: int
    """
    if len(byte_array) < 4:
        # 补零操作
        byte_array = byte_array.ljust(4, b"\x00")
    return int.from_bytes(byte_array, byteorder="little", signed=False)


def bytes_to_float(byte_array: bytearray) -> float:
    """将字节数组转换为浮点数。

    使用小端序进行转换，如果字节数组长度不足4字节，
    会在右侧补零至4字节。

    :param byte_array: 待转换的字节数组。
    :type byte_array: bytearray
    :return: 转换后的浮点数。
    :rtype: float
    """
    if len(byte_array) < 4:
        # 补零操作
        byte_array = byte_array.ljust(4, b"\x00")
    return struct.unpack("<f", byte_array)[0]


def bytes_to_spaced_hex(data: bytearray) -> str:
    """将字节数组转换为空格分隔的十六进制字符串。

    每两个十六进制字符用空格分隔，便于阅读和调试。
    例如: bytearray([0x68, 0xAA]) -> "68 aa"

    :param data: 待转换的字节数组。
    :type data: bytearray
    :return: 空格分隔的十六进制字符串。
    :rtype: str
    """
    hex_str = binascii.hexlify(data).decode("utf-8")
    # 若十六进制字符串长度为奇数，在前面补 0
    if len(hex_str) % 2 == 1:
        hex_str = "0" + hex_str
    # 每两个字符用空格分隔
    spaced_hex = " ".join([hex_str[i : i + 2] for i in range(0, len(hex_str), 2)])
    return spaced_hex


def bcd_to_string(bcd: bytearray, endian="big") -> str:
    """将 BCD 码字节数组转换为数字字符串。

    BCD（Binary-Coded Decimal）码是一种用4位二进制数表示十进制数字的编码方式。
    每个字节包含两个 BCD 数字（高4位和低4位）。

    :param bcd: BCD 码字节数组。
    :type bcd: bytearray
    :param endian: 字节序，"big" 为大端序，"little" 为小端序，默认为 "big"。
    :type endian: str
    :return: 解码后的数字字符串。
    :rtype: str
    """
    digits = []
    if endian == "little":
        bcd = bcd[::-1]
    for b in bcd:
        high = (b >> 4) & 0x0F
        low = b & 0x0F
        digits.append(str(high))
        digits.append(str(low))

    return "".join(digits)


def parse_format(format_str: str) -> tuple:
    """解析数据格式字符串，返回小数位数和总位数。

    格式字符串示例：
    - "XXXXXX.XX" -> (2, 8)  表示8位数字，2位小数
    - "XXXX.XX" -> (2, 6)    表示6位数字，2位小数
    - "XXXXXXXX" -> (0, 8)    表示8位整数，无小数

    :param format_str: 格式字符串。
    :type format_str: str
    :return: 元组 (小数位数, 总位数)。
    :rtype: tuple[int, int]
    :raises ValueError: 当格式字符串无效时抛出。
    """
    if format_str.find(".") == -1:
        return 0, len(format_str)
    parts = format_str.split(".")
    if len(parts) != 2:
        raise ValueError(f"invalid format: {format_str}")
    decimal_places = len(parts[1])
    total_digits = len(parts[0]) + len(parts[1])
    return decimal_places, total_digits


def round_float(value: float, decimal_places: int) -> float:
    """将浮点数四舍五入到指定的小数位数。

    :param value: 待四舍五入的浮点数。
    :type value: float
    :param decimal_places: 保留的小数位数。
    :type decimal_places: int
    :return: 四舍五入后的浮点数。
    :rtype: float
    """
    scale = 10**decimal_places
    return round(value * scale) / scale


def format_float(value: float, decimal_places: int, total_digits: int) -> str:
    """格式化浮点数为固定长度字符串（补零对齐）。

    :param value: 待格式化的浮点数。
    :type value: float
    :param decimal_places: 小数位数。
    :type decimal_places: int
    :param total_digits: 总字符长度（包括小数点）。
    :type total_digits: int
    :return: 格式化后的字符串。
    :rtype: str
    """
    format_str = f"{{:0{total_digits}.{decimal_places}f}}"
    return format_str.format(value)


def string_to_bcd(digits: str, endian="big") -> bytearray:
    """将数字字符串转换为 BCD 码字节数组。

    如果数字字符串长度为奇数，会在前面补零使其变为偶数长度。

    :param digits: 数字字符串（只包含 0-9）。
    :type digits: str
    :param endian: 字节序，"big" 为大端序，"little" 为小端序，默认为 "big"。
    :type endian: str
    :return: BCD 码字节数组。
    :rtype: bytearray
    """
    if len(digits) % 2 != 0:
        digits = "0" + digits  # 奇数位补零

    bcd = bytearray(len(digits) // 2)
    # 从字符串末尾开始处理（低位优先）
    for i in range(0, len(digits), 2):
        digit1 = int(digits[i])
        digit2 = int(digits[i + 1])
        byte_index = i // 2
        if endian == "little":
            byte_index = len(digits) // 2 - 1 - byte_index
        bcd[byte_index] = (digit1 << 4) | digit2
    return bytearray(bcd)


def bcd_to_value(bcd: bytearray, format_str: str, endian="big") -> Union[str, float]:
    """将 BCD 码字节数组转换为数值。

    根据格式字符串决定返回类型：
    - 如果格式字符串不包含小数点，返回数字字符串
    - 如果格式字符串包含小数点，返回浮点数

    :param bcd: BCD 码字节数组。
    :type bcd: bytearray
    :param format_str: 数据格式字符串（如 "XXXXXX.XX"）。
    :type format_str: str
    :param endian: 字节序，"big" 或 "little"，默认为 "big"。
    :type endian: str
    :return: 解析后的数值（字符串或浮点数）。
    :rtype: Union[str, float]
    """
    if format_str.find(".") == -1:
        # 无小数点格式，转换为字符串
        return bcd_to_string(bcd, endian)
    else:
        # 有小数点格式，转换为浮点数
        return bcd_to_float(bcd, format_str, endian)


def float_to_bcd(value: float, format_str: str, endian="big") -> bytearray:
    """将浮点数转换为 BCD 码字节数组。

    支持正负数转换，负数会在最高位设置符号位（0x80）。

    :param value: 待转换的浮点数。
    :type value: float
    :param format_str: 数据格式字符串（如 "XXXXXX.XX"）。
    :type format_str: str
    :param endian: 字节序，"big" 或 "little"，默认为 "big"。
    :type endian: str
    :return: BCD 码字节数组。
    :rtype: bytearray
    """
    # 判断正负
    is_negative = value < 0
    abs_value = abs(value)

    # 解析格式获取小数位数和总长度
    decimal_places, total_length = parse_format(format_str)

    if decimal_places == 0:
        bcd = string_to_bcd(format_float(abs_value, 0, total_length), endian)
    else:
        # 四舍五入并格式化为字符串
        rounded = round_float(abs_value, decimal_places)
        str_value = format_float(rounded, decimal_places, 0)  # 不需要总长度参数

        # 分离整数部分和小数部分
        integer_part, decimal_part = str_value.split(".")

        # 计算整数部分和小数部分需要的长度
        int_length = total_length - decimal_places

        # 整数部分往前补 0
        if len(integer_part) < int_length:
            integer_part = "0" * (int_length - len(integer_part)) + integer_part

        # 小数部分往后补 0
        if len(decimal_part) < decimal_places:
            decimal_part += "0" * (decimal_places - len(decimal_part))

        # 合并整数部分和小数部分并移除小数点
        digits = integer_part + decimal_part

        # 转换为BCD码
        bcd = string_to_bcd(digits, endian)

    # 设置符号位
    if is_negative and bcd:
        if endian == "big":
            bcd = bytes([bcd[0] | 0x80]) + bcd[1:]
        else:
            bcd = bcd[:-1] + bytes([bcd[-1] | 0x80])
    return bcd


def bcd_to_float(bcd, format_str, endian="big") -> float:
    """将 BCD 码字节数组转换为浮点数。

    支持带符号位的 BCD 码转换，最高位为 0x80 时表示负数。

    :param bcd: BCD 码字节数组。
    :type bcd: Union[bytearray, bytes]
    :param format_str: 数据格式字符串（如 "XXXXXX.XX"）。
    :type format_str: str
    :param endian: 字节序，"big" 或 "little"，默认为 "big"。
    :type endian: str
    :return: 解析后的浮点数。
    :rtype: float
    """
    # 检查符号位
    is_negative = False
    if bcd:
        if endian == "big":
            is_negative = (bcd[0] & 0x80) != 0
            bcd = bytes([bcd[0] & 0x7F]) + bcd[1:]
        else:
            is_negative = (bcd[-1] & 0x80) != 0
            bcd = bcd[:-1] + bytes([bcd[-1] & 0x7F])

    # 根据字节序调整BCD码顺序, 默认转换成大端序处理
    if endian == "little":
        bcd = bcd[::-1]

    # 转换为字符串
    digits = bcd_to_string(bcd)

    # 解析格式获取小数位数和总长度
    decimal_places, total_length = parse_format(format_str)

    # 若总长度不足，往前补 0
    if len(digits) < total_length:
        digits = "0" * (total_length - len(digits)) + digits

    # 分离整数部分和小数部分
    integer_part = digits[: len(digits) - decimal_places]
    decimal_part = digits[len(digits) - decimal_places :]

    # 重新组合并插入小数点
    value_str = integer_part + "." + decimal_part

    # 解析为浮点数
    value = float(value_str)
    if is_negative:
        value = -value
    return value


def datetime_to_bcd(t: datetime) -> bytearray:
    """将 datetime 对象转换为 BCD 码字节数组（小端序）。

    输出格式为：分时日月年（小端序），共5字节。
    年份只取后两位（如 2025 -> 25）。

    :param t: datetime 对象。
    :type t: datetime
    :return: BCD 码字节数组，长度为5字节。
    :rtype: bytearray
    """
    # 获取当前时间（年月日时分）
    year = t.year % 100  # 取年份后两位（如2025→25）
    month = t.month  # 月份（1-12）
    day = t.day  # 日（1-31）
    hour = t.hour  # 时（0-23）
    minute = t.minute  # 分（0-59）

    # 组合为BCD格式：YYMMDDHHmm（每个字段占1字节BCD码）
    time_bcd = bytearray(
        [
            uint8_to_bcd(year),  # 年（后两位）
            uint8_to_bcd(month),  # 月
            uint8_to_bcd(day),  # 日
            uint8_to_bcd(hour),  # 时
            uint8_to_bcd(minute),  # 分
        ]
    )
    return reverse_bytes(time_bcd)


def bcd_to_byte(b: int) -> int:
    """将单个 BCD 码字节转换为对应的十进制整数。

    例如：0x25 -> 25

    :param b: BCD 码字节（0x00-0x99）。
    :type b: int
    :return: 对应的十进制整数（0-99）。
    :rtype: int
    """
    return ((b >> 4) * 10) + (b & 0x0F)


def bcd_to_time(bcd: Union[bytearray, bytes]) -> datetime:
    """将 BCD 码字节数组转换为 datetime 对象。

    输入格式为：年月日时分（大端序），共5字节。
    年份假设为21世纪（如 25 -> 2025）。

    :param bcd: BCD 码字节数组，长度至少为5字节。
    :type bcd: Union[bytearray, bytes]
    :return: 解析后的 datetime 对象。
    :rtype: datetime
    :raises ValueError: 当 BCD 码长度不足5字节时抛出。
    """
    if len(bcd) < 5:  # 至少需要5字节（YY MM DD HH mm）
        raise ValueError("invalid BCD length")

    year = bcd_to_byte(bcd[0]) + 2000  # 假设为21世纪年份
    month = bcd_to_byte(bcd[1])
    day = bcd_to_byte(bcd[2])
    hour = bcd_to_byte(bcd[3])
    minute = bcd_to_byte(bcd[4])

    # 直接构造datetime对象
    return datetime(year, month, day, hour, minute)


def uint8_to_bcd(n: int) -> int:
    """将 0-99 范围内的整数转换为 BCD 码。

    例如：25 -> 0x25

    :param n: 0-99 范围内的整数。
    :type n: int
    :return: 对应的 BCD 码值。
    :rtype: int
    :raises ValueError: 当输入不在 0-99 范围内时抛出。
    """
    if n < 0 or n > 99:
        raise ValueError("Number must be between 0 and 99")
    return ((n // 10) << 4) | (n % 10)


def reverse_bytes(data: Union[bytearray, bytes]) -> bytearray:
    """反转字节数组的顺序。

    用于大小端序之间的转换。

    :param data: 待反转的字节数组。
    :type data: Union[bytearray, bytes]
    :return: 反转后的字节数组。
    :rtype: bytearray
    """
    return bytearray(reversed(data))




"""报文捕获数据类型模块。

本模块定义了用于实时报文捕获的数据结构：
- MessageRecord: 单条报文记录
- MessagePair: TX/RX配对记录
"""

import uuid
import time
from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from .transform import bytes_to_spaced_hex


@dataclass
class MessageRecord:
    """单条报文记录。

    用于存储发送或接收的单条报文数据。

    :ivar id: 唯一标识符 (UUID)
    :ivar direction: 报文方向，"TX"（发送）或 "RX"（接收）
    :ivar data: 原始报文数据
    :ivar timestamp: 时间戳（秒，带小数部分）
    :ivar hex_string: 十六进制字符串表示
    :ivar pair_id: 配对的报文ID，用于TX/RX关联
    """

    direction: str
    data: bytes
    id: str = field(default_factory=lambda: str(uuid.uuid4()))
    timestamp: float = field(default_factory=time.time)
    hex_string: str = field(default="")
    pair_id: Optional[str] = field(default=None)

    def __post_init__(self):
        """初始化后处理，自动生成hex_string。"""
        if not self.hex_string and self.data:
            self.hex_string = bytes_to_spaced_hex(self.data)

    def __repr__(self) -> str:
        """自定义字符串表示，显示格式化时间。"""
        return (
            f"MessageRecord(direction='{self.direction}', "
            f"time='{self.formatted_time}', "
            f"hex='{self.hex_string}')"
        )

    @property
    def formatted_time(self) -> str:
        """获取格式化的时间字符串（年-月-日 时:分:秒.毫秒）。

        :return: 格式化的时间字符串
        :rtype: str
        """
        dt = datetime.fromtimestamp(self.timestamp)
        return dt.strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]  # 截取到毫秒

    def to_dict(self) -> dict:
        """转换为字典格式。

        :return: 包含所有字段的字典
        :rtype: dict
        """
        return {
            "id": self.id,
            "direction": self.direction,
            "data": self.data.hex() if self.data else "",
            "timestamp": self.timestamp,
            "time": self.formatted_time,
            "hex_string": self.hex_string,
            "pair_id": self.pair_id,
        }


@dataclass
class MessagePair:
    """TX/RX配对记录。

    用于将发送报文和接收报文配对，便于分析请求-响应对。

    :ivar id: 配对ID
    :ivar tx: 发送报文记录
    :ivar rx: 接收报文记录
    :ivar round_trip_time: 往返时间（秒）
    """

    id: str = field(default_factory=lambda: str(uuid.uuid4()))
    tx: Optional[MessageRecord] = field(default=None)
    rx: Optional[MessageRecord] = field(default=None)
    round_trip_time: Optional[float] = field(default=None)

    def __post_init__(self):
        """初始化后处理，自动计算往返时间。"""
        self._update_round_trip_time()

    def __repr__(self) -> str:
        """自定义字符串表示，按时间顺序显示（先收到的在前）。"""
        rtt_ms = f"{abs(self.round_trip_time) * 1000:.2f}ms" if self.round_trip_time else "N/A"
        tx_hex = self.tx.hex_string if self.tx else "N/A"
        rx_hex = self.rx.hex_string if self.rx else "N/A"
        
        # 判断是服务端模式（RX先于TX）还是客户端模式（TX先于RX）
        if self.tx and self.rx and self.rx.timestamp < self.tx.timestamp:
            # 服务端模式：先RX（请求），后TX（响应）
            return (
                f"MessagePair(\n"
                f"  RX[{self.rx_time}]: {rx_hex}\n"
                f"  TX[{self.tx_time}]: {tx_hex}\n"
                f"  RTT: {rtt_ms}\n"
                f")"
            )
        else:
            # 客户端模式：先TX（请求），后RX（响应）
            return (
                f"MessagePair(\n"
                f"  TX[{self.tx_time}]: {tx_hex}\n"
                f"  RX[{self.rx_time}]: {rx_hex}\n"
                f"  RTT: {rtt_ms}\n"
                f")"
            )

    def set_tx(self, record: MessageRecord) -> None:
        """设置发送报文。

        :param record: 发送报文记录
        :type record: MessageRecord
        """
        self.tx = record
        record.pair_id = self.id
        self._update_round_trip_time()

    def set_rx(self, record: MessageRecord) -> None:
        """设置接收报文。

        :param record: 接收报文记录
        :type record: MessageRecord
        """
        self.rx = record
        record.pair_id = self.id
        self._update_round_trip_time()

    def _update_round_trip_time(self) -> None:
        """更新往返时间（始终为正值）。"""
        if self.tx is not None and self.rx is not None:
            self.round_trip_time = abs(self.tx.timestamp - self.rx.timestamp)

    def is_complete(self) -> bool:
        """检查配对是否完整（有TX和RX）。

        :return: 配对是否完整
        :rtype: bool
        """
        return self.tx is not None and self.rx is not None

    @property
    def tx_time(self) -> Optional[str]:
        """获取发送报文的格式化时间。

        :return: 格式化的发送时间，如果没有TX则返回None
        :rtype: Optional[str]
        """
        return self.tx.formatted_time if self.tx else None

    @property
    def rx_time(self) -> Optional[str]:
        """获取接收报文的格式化时间。

        :return: 格式化的接收时间，如果没有RX则返回None
        :rtype: Optional[str]
        """
        return self.rx.formatted_time if self.rx else None

    def to_dict(self) -> dict:
        """转换为字典格式。

        :return: 包含所有字段的字典
        :rtype: dict
        """
        return {
            "id": self.id,
            "tx": self.tx.to_dict() if self.tx else None,
            "rx": self.rx.to_dict() if self.rx else None,
            "tx_time": self.tx_time,
            "rx_time": self.rx_time,
            "round_trip_time": self.round_trip_time,
            "is_complete": self.is_complete(),
        }



"""设备验证模块。

本模块提供设备地址验证功能。
"""

from .types.dlt645_type import CtrlCode, PASSWORD_LEN


def validate_device(address: bytearray, ctrl_code: CtrlCode, addr: bytes) -> bool:
    """验证设备地址是否匹配。

    以下情况验证通过：
    1. 读/写通讯地址命令的响应帧（控制码带 0x80 应答标志）
    2. 广播地址（0xAA AA AA AA AA AA）
    3. 广播校时地址（0x99 99 99 99 99 99）
    4. 地址与预期地址完全匹配

    :param address: 预期的设备地址（6字节）。
    :type address: bytearray
    :param ctrl_code: 帧的控制码。
    :type ctrl_code: CtrlCode
    :param addr: 帧中的实际地址（6字节）。
    :type addr: bytes
    :return: 验证通过返回 True，否则返回 False。
    :rtype: bool
    """
    if (
        ctrl_code == CtrlCode.ReadAddress | 0x80
        or ctrl_code == CtrlCode.WriteAddress | 0x80
    ):  # 读通讯地址命令
        return True
    # 广播地址和广播时间同步地址
    if addr == bytearray([0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA]) or addr == bytearray(
        [0x99, 0x99, 0x99, 0x99, 0x99, 0x99]
    ):
        return True
    return address == addr

